# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.jwt._jwt_format."""

from absl.testing import absltest
from absl.testing import parameterized
from tink.proto import tink_pb2
from tink.jwt import _json_util
from tink.jwt import _jwt_error
from tink.jwt import _jwt_format
from tink.jwt import _raw_jwt


class JwtFormatTest(parameterized.TestCase):

  def test_base64_encode_decode_header_fixed_data(self):
    # Example from https://tools.ietf.org/html/rfc7519#section-3.1
    header = bytes([
        123, 34, 116, 121, 112, 34, 58, 34, 74, 87, 84, 34, 44, 13, 10, 32, 34,
        97, 108, 103, 34, 58, 34, 72, 83, 50, 53, 54, 34, 125
    ])
    encoded_header = b'eyJ0eXAiOiJKV1QiLA0KICJhbGciOiJIUzI1NiJ9'
    self.assertEqual(_jwt_format.base64_encode(header), encoded_header)
    self.assertEqual(_jwt_format.base64_decode(encoded_header), header)

  def test_base64_encode_decode_payload_fixed_data(self):
    # Example from https://tools.ietf.org/html/rfc7519#section-3.1
    payload = bytes([
        123, 34, 105, 115, 115, 34, 58, 34, 106, 111, 101, 34, 44, 13, 10, 32,
        34, 101, 120, 112, 34, 58, 49, 51, 48, 48, 56, 49, 57, 51, 56, 48, 44,
        13, 10, 32, 34, 104, 116, 116, 112, 58, 47, 47, 101, 120, 97, 109, 112,
        108, 101, 46, 99, 111, 109, 47, 105, 115, 95, 114, 111, 111, 116, 34,
        58, 116, 114, 117, 101, 125
    ])
    encoded_payload = (b'eyJpc3MiOiJqb2UiLA0KICJleHAiOjEzMDA4MTkzODAsDQogImh0'
                       b'dHA6Ly9leGFtcGxlLmNvbS9pc19yb290Ijp0cnVlfQ')
    self.assertEqual(_jwt_format.base64_encode(payload), encoded_payload)
    self.assertEqual(_jwt_format.base64_decode(encoded_payload), payload)

  def test_base64_decode_bad_format_raises_jwt_invalid_error(self):
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.base64_decode(b'aeyJh')

  def test_base64_decode_fails_with_unknown_chars(self):
    self.assertNotEmpty(
        _jwt_format.base64_decode(
            b'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-')
    )
    self.assertEqual(_jwt_format.base64_decode(b''), b'')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.base64_decode(b'[')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.base64_decode(b'@')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.base64_decode(b'/')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.base64_decode(b':')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.base64_decode(b'{')

  def test_decode_encode_header_hs256(self):
    # Example from https://tools.ietf.org/html/rfc7515#appendix-A.1
    encoded_header = b'eyJ0eXAiOiJKV1QiLA0KICJhbGciOiJIUzI1NiJ9'
    json_header = _jwt_format.decode_header(encoded_header)
    header = _json_util.json_loads(json_header)
    self.assertEqual(header['alg'], 'HS256')
    self.assertEqual(header['typ'], 'JWT')
    self.assertEqual(
        _jwt_format.decode_header(_jwt_format.encode_header(json_header)),
        json_header)

  def test_decode_encode_header_rs256(self):
    # Example from https://tools.ietf.org/html/rfc7515#appendix-A.2
    encoded_header = b'eyJhbGciOiJSUzI1NiJ9'
    json_header = _jwt_format.decode_header(encoded_header)
    header = _json_util.json_loads(json_header)
    self.assertEqual(header['alg'], 'RS256')
    self.assertEqual(
        _jwt_format.decode_header(_jwt_format.encode_header(json_header)),
        json_header)

  def test_encode_decode_header(self):
    encoded_header = _jwt_format.encode_header('{ "alg": "RS256"} ')
    json_header = _jwt_format.decode_header(encoded_header)
    self.assertEqual(json_header, '{ "alg": "RS256"} ')

  def test_decode_header_with_invalid_utf8(self):
    encoded_header = _jwt_format.base64_encode(b'{"alg":"RS256", "bad":"\xc2"}')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.decode_header(encoded_header)

  def test_encode_header_with_utf16_surrogate(self):
    self.assertEqual(
        _jwt_format.encode_header('{"alg": "RS256", "a":"\U0001d11e"}'),
        b'eyJhbGciOiAiUlMyNTYiLCAiYSI6IvCdhJ4ifQ')

  def test_encode_header_with_invalid_utf16_character(self):
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.encode_header('{"alg": "RS256", "a":"\uD834"}')

  @parameterized.parameters([
      'HS256', 'HS384', 'HS512', 'ES256', 'ES384', 'ES512', 'RS256', 'RS384',
      'RS384', 'RS512', 'PS256', 'PS384', 'PS512'
  ])
  def test_create_validate_header(self, algorithm):
    encoded_header = _jwt_format.create_header(algorithm, None, None)
    json_header = _jwt_format.decode_header(encoded_header)
    header = _json_util.json_loads(json_header)
    _jwt_format.validate_header(header, algorithm)
    self.assertIsNone(_jwt_format.get_type_header(header))

  def test_create_header_with_type(self):
    encoded_header = _jwt_format.create_header('HS256', 'typeHeader', None)
    json_header = _jwt_format.decode_header(encoded_header)
    self.assertEqual(json_header, '{"alg":"HS256","typ":"typeHeader"}')
    header = _json_util.json_loads(json_header)
    _jwt_format.validate_header(header, 'HS256')
    self.assertEqual(_jwt_format.get_type_header(header), 'typeHeader')

  def test_create_header_with_type_and_kid(self):
    encoded_header = _jwt_format.create_header('HS256', 'typeHeader', 'GsapRA')
    json_header = _jwt_format.decode_header(encoded_header)
    self.assertEqual(json_header,
                     '{"kid":"GsapRA","alg":"HS256","typ":"typeHeader"}')
    header = _json_util.json_loads(json_header)
    _jwt_format.validate_header(header, 'HS256')
    self.assertEqual(_jwt_format.get_type_header(header), 'typeHeader')

  def test_create_header_with_unknown_alg_fails(self):
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.create_header('unknown', None, None)

  def test_create_verify_different_algorithms_fails(self):
    encoded_header = _jwt_format.create_header('HS256', None, None)
    json_header = _jwt_format.decode_header(encoded_header)
    header = _json_util.json_loads(json_header)
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.validate_header(header, 'ES256')

  def test_verify_empty_header_fails(self):
    header = _json_util.json_loads('{}')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.validate_header(header, 'ES256')

  def test_validate_header_with_unknown_algorithm_fails(self):
    header = _json_util.json_loads('{"alg":"HS123"}')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.validate_header(header, 'HS123')

  def test_validate_header_with_unknown_entry_success(self):
    header = _json_util.json_loads('{"alg":"HS256","unknown":"header"}')
    _jwt_format.validate_header(header, 'HS256')

  def test_validate_header_ignores_typ(self):
    header = _json_util.json_loads('{"alg":"HS256","typ":"unknown"}')
    _jwt_format.validate_header(header, 'HS256')

  def test_validate_header_rejects_crit(self):
    header = _json_util.json_loads(
        '{"alg":"HS256","crit":["http://example.invalid/UNDEFINED"],'
        '"http://example.invalid/UNDEFINED":true}')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.validate_header(header, 'HS256')

  def test_validate_header_with_kid_success(self):
    json_header = '{"kid":"GsapRA","alg":"HS256"}'
    header = _json_util.json_loads(json_header)
    _jwt_format.validate_header(header, 'HS256')
    _jwt_format.validate_header(header, 'HS256', tink_kid='GsapRA')
    _jwt_format.validate_header(header, 'HS256', custom_kid='GsapRA')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      # fails because tink_kid and custom_kid must not be set at the same time.
      _jwt_format.validate_header(
          header, 'HS256', tink_kid='GsapRA', custom_kid='GsapRA')

  def test_validate_header_with_other_kid_fails(self):
    json_header = '{"kid":"GsapRA","alg":"HS256"}'
    header = _json_util.json_loads(json_header)
    _jwt_format.validate_header(header, 'HS256')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.validate_header(header, 'HS256', tink_kid='otherKid')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.validate_header(header, 'HS256', custom_kid='otherKid')

  def test_validate_header_with_missing_kid_missing(self):
    json_header = '{"alg":"HS256"}'
    header = _json_util.json_loads(json_header)
    with self.assertRaises(_jwt_error.JwtInvalidError):
      # if tink_kid is set, a kid header is required.
      _jwt_format.validate_header(header, 'HS256', tink_kid='GsapRA')
    # if custom_kid is set, a kid header is *not* required.
    _jwt_format.validate_header(header, 'HS256', custom_kid='GsapRA')

  def test_get_kid_success(self):
    key_id = 0x1ac6a944
    self.assertEqual(_jwt_format.get_kid(key_id, tink_pb2.TINK), 'GsapRA')
    self.assertIsNone(_jwt_format.get_kid(key_id, tink_pb2.RAW), None)
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.get_kid(key_id, tink_pb2.LEGACY)

  def test_get_kid_invalid_input_fails(self):
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.get_kid(123, tink_pb2.LEGACY)
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.get_kid(-1, tink_pb2.TINK)
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.get_kid(2**33, tink_pb2.TINK)

  def test_json_decode_encode_payload_fixed_data(self):
    # Example from https://tools.ietf.org/html/rfc7519#section-3.1
    encoded_payload = (b'eyJpc3MiOiJqb2UiLA0KICJleHAiOjEzMDA4MTkzODAsDQogImh0'
                       b'dHA6Ly9leGFtcGxlLmNvbS9pc19yb290Ijp0cnVlfQ')
    json_payload = _jwt_format.decode_payload(encoded_payload)
    payload = _json_util.json_loads(json_payload)
    self.assertEqual(payload['iss'], 'joe')
    self.assertEqual(payload['exp'], 1300819380)
    self.assertEqual(payload['http://example.com/is_root'], True)
    self.assertEqual(
        _jwt_format.decode_payload(_jwt_format.encode_payload(json_payload)),
        json_payload)

  def test_decode_encode_payload(self):
    # Example from https://tools.ietf.org/html/rfc7519#section-3.1
    encoded_payload = (b'eyJpc3MiOiJqb2UiLA0KICJleHAiOjEzMDA4MTkzODAsDQogImh0'
                       b'dHA6Ly9leGFtcGxlLmNvbS9pc19yb290Ijp0cnVlfQ')
    json_payload = _jwt_format.decode_payload(encoded_payload)
    payload = _json_util.json_loads(json_payload)
    self.assertEqual(payload['iss'], 'joe')
    self.assertEqual(payload['exp'], 1300819380)
    self.assertEqual(payload['http://example.com/is_root'], True)
    self.assertEqual(
        _jwt_format.decode_payload(_jwt_format.encode_payload(json_payload)),
        json_payload)

  def test_encode_payload_with_utf16_surrogate(self):
    self.assertEqual(
        _jwt_format.encode_payload('{"iss":"\U0001d11e"}'),
        b'eyJpc3MiOiLwnYSeIn0')

  def test_encode_payload_with_invalid_utf16(self):
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.encode_payload('{"iss":"\uD834"}')

  def test_create_unsigned_compact_success(self):
    raw_jwt = _raw_jwt.raw_jwt_from_json(None, '{"iss":"joe"}')
    self.assertEqual(
        _jwt_format.create_unsigned_compact('RS256', None, raw_jwt),
        b'eyJhbGciOiJSUzI1NiJ9.eyJpc3MiOiJqb2UifQ')

  def test_encode_decode_signature_success(self):
    signature = bytes([
        116, 24, 223, 180, 151, 153, 224, 37, 79, 250, 96, 125, 216, 173, 187,
        186, 22, 212, 37, 77, 105, 214, 191, 240, 91, 88, 5, 88, 83, 132, 141,
        121
    ])
    encoded = _jwt_format.encode_signature(signature)
    self.assertEqual(encoded, b'dBjftJeZ4CVP-mB92K27uhbUJU1p1r_wW1gFWFOEjXk')
    self.assertEqual(_jwt_format.decode_signature(encoded), signature)

  def test_signed_compact_create_split(self):
    raw_jwt = _raw_jwt.raw_jwt_from_json('JWT', '{"iss":"joe"}')
    signature = _jwt_format.decode_signature(
        b'dBjftJeZ4CVP-mB92K27uhbUJU1p1r_wW1gFWFOEjXk')
    unsigned_compact = _jwt_format.create_unsigned_compact(
        'RS256', None, raw_jwt)
    signed_compact = _jwt_format.create_signed_compact(unsigned_compact,
                                                       signature)
    un_comp, hdr, pay, sig = _jwt_format.split_signed_compact(signed_compact)

    self.assertEqual(
        unsigned_compact,
        b'eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJpc3MiOiJqb2UifQ')
    self.assertEqual(
        signed_compact, 'eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.'
        'eyJpc3MiOiJqb2UifQ.dBjftJeZ4CVP-mB92K27uhbUJU1p1r_wW1gFWFOEjXk')
    self.assertEqual(un_comp, unsigned_compact)
    self.assertEqual(sig, signature)
    self.assertEqual(hdr, '{"alg":"RS256","typ":"JWT"}')
    header = _json_util.json_loads(hdr)
    _jwt_format.validate_header(header, 'RS256')
    self.assertEqual(pay, '{"iss":"joe"}')
    self.assertEqual(_jwt_format.get_type_header(header), 'JWT')

  def test_signed_compact_create_split_with_kid(self):
    raw_jwt = _raw_jwt.raw_jwt_from_json(None, '{"iss":"joe"}')
    signature = _jwt_format.decode_signature(
        b'dBjftJeZ4CVP-mB92K27uhbUJU1p1r_wW1gFWFOEjXk')
    unsigned_compact = _jwt_format.create_unsigned_compact(
        'RS256', 'AZxkm2U', raw_jwt)
    signed_compact = _jwt_format.create_signed_compact(unsigned_compact,
                                                       signature)
    un_comp, hdr, pay, sig = _jwt_format.split_signed_compact(signed_compact)

    self.assertEqual(
        unsigned_compact,
        b'eyJraWQiOiJBWnhrbTJVIiwiYWxnIjoiUlMyNTYifQ.eyJpc3MiOiJqb2UifQ')
    self.assertEqual(
        signed_compact,
        'eyJraWQiOiJBWnhrbTJVIiwiYWxnIjoiUlMyNTYifQ.eyJpc3MiOiJqb2UifQ'
        '.dBjftJeZ4CVP-mB92K27uhbUJU1p1r_wW1gFWFOEjXk')
    self.assertEqual(un_comp, unsigned_compact)
    self.assertEqual(sig, signature)
    self.assertEqual(hdr, '{"kid":"AZxkm2U","alg":"RS256"}')
    header = _json_util.json_loads(hdr)
    _jwt_format.validate_header(header, 'RS256')
    self.assertEqual(pay, '{"iss":"joe"}')
    self.assertIsNone(_jwt_format.get_type_header(header))

  def test_split_empty_signed_compact(self):
    un_comp, hdr, pay, sig = _jwt_format.split_signed_compact('..')
    self.assertEqual(un_comp, b'.')
    self.assertEmpty(sig)
    self.assertEmpty(hdr)
    self.assertEmpty(pay)

  def test_split_signed_compact_success(self):
    un_comp, hdr, pay, sig = _jwt_format.split_signed_compact('e30.e30.YWJj')
    self.assertEqual(un_comp, b'e30.e30')
    self.assertEqual(sig, b'abc')
    self.assertEqual(hdr, '{}')
    self.assertEqual(pay, '{}')

  def test_split_signed_compact_with_bad_format_fails(self):
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('e30.e30.YWJj.abc')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('e30.e30.YWJj.')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('.e30.e30.YWJj')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('.e30.e30.')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('e30')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('')

  def test_split_signed_compact_with_bad_characters_fails(self):
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('{e30.e30.YWJj')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact(' e30.e30.YWJj')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('e30. e30.YWJj')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('e30.e30.YWJj ')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('e30.e30.\nYWJj')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('e30.\re30.YWJj')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('e30$.e30.YWJj')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('e30.$e30.YWJj')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('e30.e30.YWJj$')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact('e30.e30.YWJj\ud83c')

  def test_split_signed_compact_with_invalid_utf8_in_header(self):
    encoded_header = _jwt_format.base64_encode(b'{"alg":"RS256", "bad":"\xc2"}')
    token = (encoded_header + b'.e30.YWJj').decode('utf8')
    with self.assertRaises(_jwt_error.JwtInvalidError):
      _jwt_format.split_signed_compact(token)


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.streaming_aead.streaming_aead."""

import io
import os
import tempfile

from absl.testing import absltest

import tink
from tink import streaming_aead


def setUpModule():
  streaming_aead.register()


def get_primitive() -> streaming_aead.StreamingAead:
  key_template = streaming_aead.streaming_aead_key_templates.AES128_GCM_HKDF_4KB
  keyset_handle = tink.new_keyset_handle(key_template)
  primitive = keyset_handle.primitive(streaming_aead.StreamingAead)
  return primitive


class StreamingAeadTest(absltest.TestCase):
  """End-to-end test of Streaming AEAD Encrypting/Decrypting Streams."""

  def test_encrypt_decrypt(self):
    primitive = get_primitive()
    long_plaintext = b' '.join(b'%d' % i for i in range(100 * 1000))
    aad = b'associated_data'
    with tempfile.TemporaryDirectory() as tmpdirname:
      filename = os.path.join(tmpdirname, 'encrypted_file')
      dest = open(filename, 'wb')
      with primitive.new_encrypting_stream(dest, aad) as es:
        n = es.write(long_plaintext)
      self.assertTrue(dest.closed)
      self.assertLen(long_plaintext, n)

      src = open(filename, 'rb')
      with primitive.new_decrypting_stream(src, aad) as ds:
        output = ds.read()
      self.assertTrue(src.closed)
      self.assertEqual(output, long_plaintext)

  def test_encrypt_decrypt_raw(self):
    primitive = get_primitive()
    long_plaintext = b' '.join(b'%d' % i for i in range(100 * 1000))
    aad = b'associated_data'
    with tempfile.TemporaryDirectory() as tmpdirname:
      filename = os.path.join(tmpdirname, 'encrypted_file_raw')
      dest = open(filename, 'wb', buffering=0)  # returns a raw file.
      with primitive.new_encrypting_stream(dest, aad) as es:
        n = es.write(long_plaintext)
      self.assertTrue(dest.closed)
      self.assertLen(long_plaintext, n)

      src = open(filename, 'rb', buffering=0)  # returns a raw file.
      with primitive.new_decrypting_stream(src, aad) as ds:
        output = ds.read()
      self.assertTrue(src.closed)
      self.assertEqual(output, long_plaintext)

  def test_encrypt_decrypt_textiowrapper(self):
    primitive = get_primitive()
    text_lines = [
        'ᚻᛖ ᚳᚹᚫᚦ ᚦᚫᛏ ᚻᛖ ᛒᚢᛞᛖ ᚩᚾ ᚦᚫᛗ ᛚᚪᚾᛞᛖ ᚾᚩᚱᚦᚹᛖᚪᚱᛞᚢᛗ ᚹᛁᚦ ᚦᚪ ᚹᛖᛥᚫ\n',
        '⡌⠁⠧⠑ ⠼⠁⠒  ⡍⠜⠇⠑⠹⠰⠎ ⡣⠕⠌\n',
        '2H₂ + O₂ ⇌ 2H₂O\n',
        'smile 😀\n']
    aad = b'associated_data'
    with tempfile.TemporaryDirectory() as tmpdirname:
      filename = os.path.join(tmpdirname, 'encrypted_textfile')
      dest = open(filename, 'wb')
      with io.TextIOWrapper(
          primitive.new_encrypting_stream(dest, aad), encoding='utf8') as es:
        es.writelines(text_lines)
      self.assertTrue(dest.closed)

      src = open(filename, 'rb')
      with io.TextIOWrapper(
          primitive.new_decrypting_stream(src, aad), encoding='utf8') as es:
        for i, text_line in enumerate(es):
          self.assertEqual(text_line, text_lines[i])
      self.assertTrue(src.closed)

  def test_encrypt_fails_on_nonwritable_stream(self):
    primitive = get_primitive()
    with tempfile.TemporaryDirectory() as tmpdirname:
      filename = os.path.join(tmpdirname, 'file')
      with open(filename, 'wb') as f:
        f.write(b'data')
      with open(filename, 'rb') as dest:  # dest is not writable
        with self.assertRaises(ValueError):
          primitive.new_encrypting_stream(dest, b'aad')

  def test_decrypt_fails_on_nonreadable_stream(self):
    primitive = get_primitive()
    with tempfile.TemporaryDirectory() as tmpdirname:
      # src not readable
      with open(os.path.join(tmpdirname, 'file2'), 'wb') as src:
        with self.assertRaises(ValueError):
          primitive.new_decrypting_stream(src, b'aad')

if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Python wrapper of the wrapped C++ PRF Set key manager."""

from tink import core
from tink.cc.pybind import tink_bindings
from tink.prf import _prf_set
from tink.prf import _prf_set_wrapper


class PrfCcToPyWrapper(_prf_set.Prf):
  """Transforms C++ Prf primitive into a Python Prf primitive."""

  def __init__(self, cc_primitive: tink_bindings.Prf):
    self._cc_primitive = cc_primitive

  @core.use_tink_errors
  def compute(self, input_data: bytes, output_length: int) -> bytes:
    return self._cc_primitive.compute(input_data, output_length)


def register() -> None:
  """Registers all PrfSet key managers and PrfSet wrapper in the Registry."""
  tink_bindings.register()
  for ident in (
      'AesCmacPrfKey',
      'HmacPrfKey',
      'HkdfPrfKey',
  ):
    type_url = 'type.googleapis.com/google.crypto.tink.{}'.format(ident)
    key_manager = core.KeyManagerCcToPyWrapper(
        tink_bindings.PrfKeyManager.from_cc_registry(type_url),
        _prf_set.Prf, PrfCcToPyWrapper)
    core.Registry.register_key_manager(key_manager, new_key_allowed=True)
  core.Registry.register_primitive_wrapper(_prf_set_wrapper.PrfSetWrapper())



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This module defines the interface for Deterministic AEAD."""

import abc


class DeterministicAead(metaclass=abc.ABCMeta):
  """Interface for Deterministic Authenticated Encryption with Associated Data.

  For why this interface is desirable and some of its use cases, see for
  example https://tools.ietf.org/html/rfc5297#section-1.3.

  Warning!

  Unlike Aead, implementations of this interface are not semantically
  secure, because encrypting the same plaintex always yields the same
  ciphertext.

  Security guarantees

  Implementations of this interface provide 128-bit security level against
  multi-user attacks with up to 2^32 keys. That means if an adversary
  obtains 2^32 ciphertexts of the same message encrypted under 2^32 keys,
  they need to do 2^128 computations to obtain a single key.

  Encryption with associated data ensures authenticity (who the sender is)
  and integrity (the data has not been tampered with) of that data, but not
  its secrecy. (see https://tools.ietf.org/html/rfc5116)
  """

  @abc.abstractmethod
  def encrypt_deterministically(self, plaintext: bytes,
                                associated_data: bytes) -> bytes:
    raise NotImplementedError()

  @abc.abstractmethod
  def decrypt_deterministically(self, ciphertext: bytes,
                                associated_data: bytes) -> bytes:
    raise NotImplementedError()



# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for kms_aead_key_manager."""

from absl.testing import absltest
from tink.proto import kms_aead_pb2
from tink.proto import kms_envelope_pb2
from tink.proto import tink_pb2
import tink
from tink import aead
from tink.aead import _kms_aead_key_manager


def setUpModule():
  aead.register()


class FakeClient(tink.KmsClient):

  def __init__(self, key_uri):
    self.key_uri = key_uri

  def does_support(self, key_uri: str) -> bool:
    return key_uri == self.key_uri

  def get_aead(self, key_uri: str) -> aead.Aead:
    raise NotImplementedError('not implemented')


KMS_AEAD_KEY_TYPE_URL = 'type.googleapis.com/google.crypto.tink.KmsAeadKey'
KMS_ENVELOPE_AEAD_KEY_TYPE_URL = (
    'type.googleapis.com/google.crypto.tink.KmsEnvelopeAeadKey'
)


class KmsAeadKeyManagerTest(absltest.TestCase):

  def test_kms_aead_new_key_data_success(self):
    client = FakeClient('key_uri')
    tink.register_kms_client(client)
    template = aead.aead_key_templates.create_kms_aead_key_template(
        key_uri='key_uri'
    )
    key_data = _kms_aead_key_manager.KmsAeadKeyManager().new_key_data(template)

    self.assertEqual(key_data.type_url, KMS_AEAD_KEY_TYPE_URL)
    self.assertEqual(key_data.key_material_type, tink_pb2.KeyData.REMOTE)
    key = kms_aead_pb2.KmsAeadKey.FromString(key_data.value)
    self.assertEqual(key.version, 0)
    self.assertEqual(key.params.key_uri, 'key_uri')

  def test_kms_aead_new_key_data_rejects_unknown_template(self):
    with self.assertRaises(tink.TinkError):
      _kms_aead_key_manager.KmsAeadKeyManager().new_key_data(
          aead.aead_key_templates.XCHACHA20_POLY1305_RAW
      )

  def test_kms_aead_primitive_rejects_unknown_key_data(self):
    template = aead.aead_key_templates.create_kms_envelope_aead_key_template(
        kek_uri='kek_uri',
        dek_template=aead.aead_key_templates.XCHACHA20_POLY1305_RAW,
    )
    envelope_aead_key_data = (
        _kms_aead_key_manager.KmsEnvelopeAeadKeyManager().new_key_data(template)
    )

    with self.assertRaises(tink.TinkError):
      _kms_aead_key_manager.KmsAeadKeyManager().primitive(
          envelope_aead_key_data
      )

  def test_kms_envelope_aead_new_key_data_success(self):
    template = aead.aead_key_templates.create_kms_envelope_aead_key_template(
        kek_uri='kek_uri',
        dek_template=aead.aead_key_templates.XCHACHA20_POLY1305_RAW,
    )
    key_data = _kms_aead_key_manager.KmsEnvelopeAeadKeyManager().new_key_data(
        template
    )

    self.assertEqual(key_data.type_url, KMS_ENVELOPE_AEAD_KEY_TYPE_URL)
    self.assertEqual(key_data.key_material_type, tink_pb2.KeyData.REMOTE)
    key = kms_envelope_pb2.KmsEnvelopeAeadKey.FromString(key_data.value)
    self.assertEqual(key.version, 0)
    self.assertEqual(key.params.kek_uri, 'kek_uri')
    self.assertEqual(
        key.params.dek_template, aead.aead_key_templates.XCHACHA20_POLY1305_RAW
    )

  def test_kms_envelope_aead_rejects_unknown_template(self):
    with self.assertRaises(tink.TinkError):
      _kms_aead_key_manager.KmsEnvelopeAeadKeyManager().new_key_data(
          aead.aead_key_templates.XCHACHA20_POLY1305_RAW
      )

  def test_kms_envelope_aead_primitive_rejects_unknown_key_data(self):
    template = aead.aead_key_templates.create_kms_aead_key_template(
        key_uri='key_uri'
    )
    kms_aead_key_data = _kms_aead_key_manager.KmsAeadKeyManager().new_key_data(
        template
    )

    with self.assertRaises(tink.TinkError):
      _kms_aead_key_manager.KmsEnvelopeAeadKeyManager().primitive(
          kms_aead_key_data
      )


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""AEAD wrapper."""

from typing import Optional, Type

from tink import _monitoring
from tink import core
from tink.aead import _aead


class _WrappedAead(_aead.Aead):
  """Implements Aead for a set of Aead primitives."""

  def __init__(
      self,
      pset: core.PrimitiveSet,
      encryption_monitor: Optional[_monitoring.KeyUsageMonitor] = None,
      decryption_monitor: Optional[_monitoring.KeyUsageMonitor] = None,
  ):
    self._primitive_set = pset
    self._encryption_monitor = encryption_monitor
    self._decryption_monitor = decryption_monitor

  def encrypt(self, plaintext: bytes, associated_data: bytes) -> bytes:
    primary = self._primitive_set.primary()
    result = primary.identifier + primary.primitive.encrypt(
        plaintext, associated_data
    )

    if self._encryption_monitor:
      self._encryption_monitor.log(primary.key_id, len(plaintext))

    return result

  def decrypt(self, ciphertext: bytes, associated_data: bytes) -> bytes:
    if len(ciphertext) > core.crypto_format.NON_RAW_PREFIX_SIZE:
      prefix = ciphertext[: core.crypto_format.NON_RAW_PREFIX_SIZE]
      ciphertext_no_prefix = ciphertext[
          core.crypto_format.NON_RAW_PREFIX_SIZE :
      ]
      for entry in self._primitive_set.primitive_from_identifier(prefix):
        try:
          result = entry.primitive.decrypt(
              ciphertext_no_prefix, associated_data
          )
          if self._decryption_monitor:
            self._decryption_monitor.log(
                entry.key_id, len(ciphertext_no_prefix)
            )
          return result
        except core.TinkError:
          pass
    # Let's try all RAW keys.
    for entry in self._primitive_set.raw_primitives():
      try:
        result = entry.primitive.decrypt(ciphertext, associated_data)
        if self._decryption_monitor:
          self._decryption_monitor.log(entry.key_id, len(ciphertext))
        return result
      except core.TinkError:
        pass

    # nothing works.
    if self._decryption_monitor:
      self._decryption_monitor.log_failure()

    raise core.TinkError('Decryption failed.')


class AeadWrapper(core.PrimitiveWrapper[_aead.Aead, _aead.Aead]):
  """AeadWrapper is the implementation of PrimitiveWrapper for Aead.

  Key rotation works as follows: each ciphertext is prefixed with the keyId.
  When decrypting, we first try all primitives whose keyId starts with the
  prefix of the ciphertext. If none of these succeed, we try the raw primitives.
  If any succeeds, we return the ciphertext, otherwise we simply raise a
  TinkError.
  """

  def wrap(self, pset: core.PrimitiveSet) -> _aead.Aead:
    return _WrappedAead(pset)

  def primitive_class(self) -> Type[_aead.Aead]:
    return _aead.Aead

  def input_primitive_class(self) -> Type[_aead.Aead]:
    return _aead.Aead

  def _wrap_with_monitoring_info(
      self,
      pset: core.PrimitiveSet,
      monitoring_keyset_info: _monitoring.MonitoringKeySetInfo,
  ) -> _aead.Aead:
    encryption_key_usage_monitor = _monitoring.get_key_usage_monitor_or_none(
        _monitoring.MonitoringContext(
            primitive='aead',
            api_function='encrypt',
            keyset_info=monitoring_keyset_info,
        )
    )
    decryption_key_usage_monitor = _monitoring.get_key_usage_monitor_or_none(
        _monitoring.MonitoringContext(
            primitive='aead',
            api_function='decrypt',
            keyset_info=monitoring_keyset_info,
        )
    )
    return _WrappedAead(
        pset, encryption_key_usage_monitor, decryption_key_usage_monitor
    )



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.integration.aws_kms_client."""

import os
import tempfile

from absl.testing import absltest
import boto3

import tink
from tink.integration import awskms
from tink.integration.awskms import _aws_kms_client
from tink.testing import helper


CREDENTIAL_PATH = os.path.join(
    helper.tink_py_testdata_path(), 'aws/credentials.ini'
)

KEY_URI = (
    'aws-kms://arn:aws:kms:us-east-2:235739564943:key/'
    '3ee50705-5a82-4f5b-9753-05c4f473922f'
)

# An alias for KEY_URI.
KEY_ALIAS_URI = (
    'aws-kms://arn:aws:kms:us-east-2:235739564943:alias/'
    'unit-and-integration-testing'
)

KEY_URI_2 = (
    'aws-kms://arn:aws:kms:us-east-2:235739564943:key/'
    'b3ca2efd-a8fb-47f2-b541-7e20f8c5cd11'
)

GCP_KEY_URI = (
    'gcp-kms://projects/tink-test-infrastructure/locations/global/'
    'keyRings/unit-and-integration-testing/cryptoKeys/aead-key'
)


class AwsKmsClientTest(absltest.TestCase):

  def test_has_aws_key_uri_format_is_true_for_valid_key_uri(self):
    self.assertTrue(_aws_kms_client._has_aws_key_uri_format(KEY_URI))
    self.assertTrue(_aws_kms_client._has_aws_key_uri_format(KEY_ALIAS_URI))
    self.assertTrue(_aws_kms_client._has_aws_key_uri_format(KEY_URI_2))

  def test_has_aws_key_uri_format_is_false_for_invalid_key_uri(self):
    # a GCP key URI is not a valid AWS key URI.
    self.assertFalse(_aws_kms_client._has_aws_key_uri_format(GCP_KEY_URI))
    # key URI is case sensitive
    self.assertFalse(
        _aws_kms_client._has_aws_key_uri_format(
            'aws-kms://arn:AWS:kms:us-east-2:235739564943:key/'
            '3ee50705-5a82-4f5b-9753-05c4f473922f'
        )
    )

  def test_key_uri_to_key_arn(self):
    self.assertEqual(
        _aws_kms_client._key_uri_to_key_arn(
            'aws-kms://arn:aws:kms:us-east-2:235739564943:key/'
            '3ee50705-5a82-4f5b-9753-05c4f473922f'
        ),
        'arn:aws:kms:us-east-2:235739564943:key/'
        '3ee50705-5a82-4f5b-9753-05c4f473922f',
    )

  def test_get_region_from_key_arn(self):
    self.assertEqual(
        _aws_kms_client._get_region_from_key_arn(
            'arn:aws:kms:us-east-2:235739564943:key/'
            '3ee50705-5a82-4f5b-9753-05c4f473922f'
        ),
        'us-east-2',
    )

  def test_client_bound_to_key_uri(self):
    aws_client = awskms.AwsKmsClient(KEY_URI, CREDENTIAL_PATH)

    self.assertEqual(aws_client.does_support(KEY_URI), True)
    self.assertEqual(aws_client.does_support(KEY_ALIAS_URI), False)
    self.assertEqual(aws_client.does_support(KEY_URI_2), False)
    self.assertEqual(aws_client.does_support(GCP_KEY_URI), False)

  def test_client_not_bound_to_key_uri(self):
    aws_client = awskms.AwsKmsClient('', CREDENTIAL_PATH)

    self.assertEqual(aws_client.does_support(KEY_URI), True)
    self.assertEqual(aws_client.does_support(KEY_ALIAS_URI), True)
    self.assertEqual(aws_client.does_support(KEY_URI_2), True)
    self.assertEqual(aws_client.does_support(GCP_KEY_URI), False)

  def test_unbound_new_client_does_the_same_as_aws_kms_client(self):
    # create client that is not bound to a key URI using AwsKmsClient
    want_aws_client = awskms.AwsKmsClient(None, CREDENTIAL_PATH)
    # create client that is not bound to a key URI using new_client
    aws_access_key_id, aws_secret_access_key = _aws_kms_client._parse_config(
        CREDENTIAL_PATH
    )
    boto3_client = boto3.client(
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        region_name='us-east-2',
        service_name='kms',
    )
    aws_client = awskms.new_client(boto3_client=boto3_client)

    self.assertEqual(
        aws_client.does_support(KEY_URI), want_aws_client.does_support(KEY_URI)
    )
    self.assertEqual(
        aws_client.does_support(KEY_ALIAS_URI),
        want_aws_client.does_support(KEY_ALIAS_URI),
    )
    self.assertEqual(
        aws_client.does_support(KEY_URI_2),
        want_aws_client.does_support(KEY_URI_2),
    )
    self.assertEqual(
        aws_client.does_support(GCP_KEY_URI),
        want_aws_client.does_support(GCP_KEY_URI),
    )
    self.assertEqual(
        aws_client.does_support(''), want_aws_client.does_support('')
    )

  def test_bound_new_client_does_the_same_as_aws_kms_client(self):
    # create client that is bound to a key URI using AwsKmsClient
    want_aws_client = awskms.AwsKmsClient(KEY_URI, CREDENTIAL_PATH)
    # create client that is bound to a key URI  using new_client
    aws_access_key_id, aws_secret_access_key = _aws_kms_client._parse_config(
        CREDENTIAL_PATH
    )
    boto3_client = boto3.client(
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        region_name='us-east-2',
        service_name='kms',
    )
    aws_client = awskms.new_client(boto3_client=boto3_client, key_uri=KEY_URI)

    self.assertEqual(
        aws_client.does_support(KEY_URI), want_aws_client.does_support(KEY_URI)
    )
    self.assertEqual(
        aws_client.does_support(KEY_ALIAS_URI),
        want_aws_client.does_support(KEY_ALIAS_URI),
    )
    self.assertEqual(
        aws_client.does_support(KEY_URI_2),
        want_aws_client.does_support(KEY_URI_2),
    )
    self.assertEqual(
        aws_client.does_support(GCP_KEY_URI),
        want_aws_client.does_support(GCP_KEY_URI),
    )
    self.assertEqual(
        aws_client.does_support(''), want_aws_client.does_support('')
    )

  def test_invalid_key_uri(self):
    with self.assertRaises(tink.TinkError):
      awskms.AwsKmsClient(GCP_KEY_URI, CREDENTIAL_PATH)

  def test_client_empty_key_uri(self):
    aws_client = awskms.AwsKmsClient('', CREDENTIAL_PATH)
    self.assertEqual(aws_client.does_support(KEY_URI), True)

  def test_client_invalid_path(self):
    with self.assertRaises(ValueError):
      awskms.AwsKmsClient('', CREDENTIAL_PATH + 'corrupted')

  def test_wrong_credentials_path(self):
    with self.assertRaises(ValueError):
      awskms.AwsKmsClient(KEY_URI, '../credentials.txt')

  def test_parse_valid_credentials_works(self):
    config_file = tempfile.NamedTemporaryFile(delete=False)
    with open(config_file.name, 'w') as f:
      f.write("""
[otherSection]
aws_access_key_id = other_key_id
aws_secret_access_key = other_key

[default]
aws_access_key_id = key_id_123
aws_secret_access_key = key_123""")

    aws_access_key_id, aws_secret_access_key = _aws_kms_client._parse_config(
        config_file.name
    )
    self.assertEqual(aws_access_key_id, 'key_id_123')
    self.assertEqual(aws_secret_access_key, 'key_123')

  def test_parse_credentials_without_key_id_fails(self):
    config_file = tempfile.NamedTemporaryFile(delete=False)
    with open(config_file.name, 'w') as f:
      f.write("""
[default]
aws_secret_access_key = key_123""")
    with self.assertRaises(ValueError):
      _aws_kms_client._parse_config(config_file.name)

  def test_parse_credentials_without_key_fails(self):
    config_file = tempfile.NamedTemporaryFile(delete=False)
    with open(config_file.name, 'w') as f:
      f.write("""
[default]
aws_secret_access_key = key_123""")
    with self.assertRaises(ValueError):
      _aws_kms_client._parse_config(config_file.name)

  def test_parse_credentials_without_default_section_fails(self):
    config_file = tempfile.NamedTemporaryFile(delete=False)
    with open(config_file.name, 'w') as f:
      f.write("""
[otherSection]
aws_access_key_id = other_key_id
aws_secret_access_key = other_key""")
    with self.assertRaises(ValueError):
      _aws_kms_client._parse_config(config_file.name)


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This module defines the interface for PrfSet."""

import abc
from typing import Mapping


class Prf(metaclass=abc.ABCMeta):
  """An element of a pseudo random function family, selected by a key.

  The PRF interface is an abstraction for an element of a pseudo random function
  family, selected by a key. It has the following properties:

  - It is deterministic: Prf.compute(input, length) will always return the same
    output if the same key is used. Prf.compute(input, length1) will be a prefix
    of Prf.compute(input, length2) if length1 < length2 and the same key is
    used.

  - It is indistinguishable from a random function: Given the evaluation of n
    different inputs, an attacker cannot distinguish between the PRF and random
    bytes on an input different from the n that are known.

  Use cases for PRF are deterministic redaction of PII, keyed hash functions,
  creating sub IDs that do not allow joining with the original dataset without
  knowing the key. While PRFs can be used in order to prove authenticity of a
  message, using the MAC interface is recommended for that use case, as it has
  support for verification, avoiding the security problems that often happen
  during verification. It also allows for non-deterministic MAC algorithms.
  """

  @abc.abstractmethod
  def compute(self, input_data: bytes, output_length: int) -> bytes:
    """Computes the PRF selected by the underlying key.

    Args:
      input_data: The input to compute the PRF on.
      output_length: The desired length of the output in bytes. When choosing
        this parameter keep the birthday paradox in mind. If you have 2^n
        different inputs that your system has to handle set the output length to
        ceil(n/4 + 4) This corresponds to 2*n + 32 bits, meaning a
          collision will occur with a probability less than 1:2^32. When in
            doubt, request a security review.

    Returns:
      the first output_length bytes of the PRF.
    """
    raise NotImplementedError()


class PrfSet(metaclass=abc.ABCMeta):
  """A Tink Keyset can be converted into a set of PRFs using this primitive.

  Every key in the keyset corresponds to a PRF in the PrfSet. Every PRF in the
  set is given an ID, which is the same ID as the key id in the Keyset.
  """

  @abc.abstractmethod
  def primary_id(self) -> int:
    raise NotImplementedError()

  @abc.abstractmethod
  def all(self) -> Mapping[int, Prf]:
    raise NotImplementedError()

  @abc.abstractmethod
  def primary(self) -> Prf:
    raise NotImplementedError()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/hpke.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/hpke.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15tink/proto/hpke.proto\x12\x12google.crypto.tink\"\x8c\x01\n\nHpkeParams\x12(\n\x03kem\x18\x01 \x01(\x0e\x32\x1b.google.crypto.tink.HpkeKem\x12(\n\x03kdf\x18\x02 \x01(\x0e\x32\x1b.google.crypto.tink.HpkeKdf\x12*\n\x04\x61\x65\x61\x64\x18\x03 \x01(\x0e\x32\x1c.google.crypto.tink.HpkeAead\"d\n\rHpkePublicKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12.\n\x06params\x18\x02 \x01(\x0b\x32\x1e.google.crypto.tink.HpkeParams\x12\x12\n\npublic_key\x18\x03 \x01(\x0c\"m\n\x0eHpkePrivateKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x35\n\npublic_key\x18\x02 \x01(\x0b\x32!.google.crypto.tink.HpkePublicKey\x12\x13\n\x0bprivate_key\x18\x03 \x01(\x0c\"?\n\rHpkeKeyFormat\x12.\n\x06params\x18\x01 \x01(\x0b\x32\x1e.google.crypto.tink.HpkeParams*\xb7\x01\n\x07HpkeKem\x12\x0f\n\x0bKEM_UNKNOWN\x10\x00\x12\x1c\n\x18\x44HKEM_X25519_HKDF_SHA256\x10\x01\x12\x1a\n\x16\x44HKEM_P256_HKDF_SHA256\x10\x02\x12\x1a\n\x16\x44HKEM_P384_HKDF_SHA384\x10\x03\x12\x1a\n\x16\x44HKEM_P521_HKDF_SHA512\x10\x04\x12\n\n\x06X_WING\x10\x05\x12\r\n\tML_KEM768\x10\x06\x12\x0e\n\nML_KEM1024\x10\x07*M\n\x07HpkeKdf\x12\x0f\n\x0bKDF_UNKNOWN\x10\x00\x12\x0f\n\x0bHKDF_SHA256\x10\x01\x12\x0f\n\x0bHKDF_SHA384\x10\x02\x12\x0f\n\x0bHKDF_SHA512\x10\x03*U\n\x08HpkeAead\x12\x10\n\x0c\x41\x45\x41\x44_UNKNOWN\x10\x00\x12\x0f\n\x0b\x41\x45S_128_GCM\x10\x01\x12\x0f\n\x0b\x41\x45S_256_GCM\x10\x02\x12\x15\n\x11\x43HACHA20_POLY1305\x10\x03\x42T\n\x1c\x63om.google.crypto.tink.protoP\x01Z2github.com/tink-crypto/tink-go/v2/proto/hpke_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.hpke_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z2github.com/tink-crypto/tink-go/v2/proto/hpke_proto'
  _globals['_HPKEKEM']._serialized_start=467
  _globals['_HPKEKEM']._serialized_end=650
  _globals['_HPKEKDF']._serialized_start=652
  _globals['_HPKEKDF']._serialized_end=729
  _globals['_HPKEAEAD']._serialized_start=731
  _globals['_HPKEAEAD']._serialized_end=816
  _globals['_HPKEPARAMS']._serialized_start=46
  _globals['_HPKEPARAMS']._serialized_end=186
  _globals['_HPKEPUBLICKEY']._serialized_start=188
  _globals['_HPKEPUBLICKEY']._serialized_end=288
  _globals['_HPKEPRIVATEKEY']._serialized_start=290
  _globals['_HPKEPRIVATEKEY']._serialized_end=399
  _globals['_HPKEKEYFORMAT']._serialized_start=401
  _globals['_HPKEKEYFORMAT']._serialized_end=464
# @@protoc_insertion_point(module_scope)



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.streaming_aead_key_templates."""

from absl.testing import absltest
from absl.testing import parameterized
from tink.proto import aes_ctr_hmac_streaming_pb2
from tink.proto import aes_gcm_hkdf_streaming_pb2
from tink.proto import common_pb2
from tink.proto import tink_pb2
from tink import streaming_aead


class StreamingAeadKeyTemplatesTest(parameterized.TestCase):

  def test_create_aes_gcm_hkdf_streaming_key_template(self):
    # Intentionally using 'weird' or invalid values for parameters,
    # to test that the function correctly puts them in the resulting template.
    template = None
    with self.assertWarns(DeprecationWarning):
      template = (
          streaming_aead.streaming_aead_key_templates
          .create_aes_gcm_hkdf_streaming_key_template(
              aes_key_size=42,
              hash_type=common_pb2.HashType.SHA1,
              derived_key_size=76,
              ciphertext_segment_size=64,
          ))
    self.assertEqual(
        'type.googleapis.com/google.crypto.tink.AesGcmHkdfStreamingKey',
        template.type_url)
    self.assertEqual(tink_pb2.RAW, template.output_prefix_type)
    key_format = aes_gcm_hkdf_streaming_pb2.AesGcmHkdfStreamingKeyFormat()
    key_format.ParseFromString(template.value)
    self.assertEqual(42, key_format.key_size)
    self.assertEqual(common_pb2.HashType.SHA1, key_format.params.hkdf_hash_type)
    self.assertEqual(76, key_format.params.derived_key_size)
    self.assertEqual(64, key_format.params.ciphertext_segment_size)

  def test_create_aes_ctr_hmac_streaming_key_template(self):
    # Intentionally using 'weird' or invalid values for parameters,
    # to test that the function correctly puts them in the resulting template.
    template = None
    with self.assertWarns(DeprecationWarning):
      template = (
          streaming_aead.streaming_aead_key_templates
          .create_aes_ctr_hmac_streaming_key_template(
              aes_key_size=42,
              hkdf_hash_type=common_pb2.HashType.SHA1,
              derived_key_size=76,
              mac_hash_type=common_pb2.HashType.UNKNOWN_HASH,
              tag_size=39,
              ciphertext_segment_size=64,
          ))
    self.assertEqual(
        'type.googleapis.com/google.crypto.tink.AesCtrHmacStreamingKey',
        template.type_url)
    self.assertEqual(tink_pb2.RAW, template.output_prefix_type)
    key_format = aes_ctr_hmac_streaming_pb2.AesCtrHmacStreamingKeyFormat()
    key_format.ParseFromString(template.value)
    self.assertEqual(42, key_format.key_size)
    self.assertEqual(common_pb2.HashType.SHA1, key_format.params.hkdf_hash_type)
    self.assertEqual(76, key_format.params.derived_key_size)
    self.assertEqual(common_pb2.HashType.UNKNOWN_HASH,
                     key_format.params.hmac_params.hash)
    self.assertEqual(39, key_format.params.hmac_params.tag_size)
    self.assertEqual(64, key_format.params.ciphertext_segment_size)


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Python wrapper of the wrapped C++ Public Key Signature key manager."""

from tink import core
from tink.cc.pybind import tink_bindings
from tink.signature import _public_key_sign
from tink.signature import _public_key_verify
from tink.signature import _signature_wrapper


class _PublicKeySignCcToPyWrapper(_public_key_sign.PublicKeySign):
  """Transforms C++ PublicKeySign into a Python primitive."""

  def __init__(self, cc_primitive: tink_bindings.PublicKeySign):
    self._public_key_sign = cc_primitive

  @core.use_tink_errors
  def sign(self, data: bytes) -> bytes:
    return self._public_key_sign.sign(data)


class _PublicKeyVerifyCcToPyWrapper(_public_key_verify.PublicKeyVerify):
  """Transforms C++ PublicKeyVerify into a Python primitive."""

  def __init__(self, cc_primitive: tink_bindings.PublicKeyVerify):
    self._public_key_verify = cc_primitive

  @core.use_tink_errors
  def verify(self, signature: bytes, data: bytes) -> None:
    self._public_key_verify.verify(signature, data)


def register():
  """Registers all signature key managers in the Python registry."""
  tink_bindings.register()

  for key_type_identifier in ('EcdsaPrivateKey', 'Ed25519PrivateKey',
                              'RsaSsaPssPrivateKey', 'RsaSsaPkcs1PrivateKey',):
    type_url = 'type.googleapis.com/google.crypto.tink.' + key_type_identifier
    key_manager = core.PrivateKeyManagerCcToPyWrapper(
        tink_bindings.PublicKeySignKeyManager.from_cc_registry(type_url),
        _public_key_sign.PublicKeySign, _PublicKeySignCcToPyWrapper)
    core.Registry.register_key_manager(key_manager, new_key_allowed=True)

  for key_type_identifier in ('EcdsaPublicKey', 'Ed25519PublicKey',
                              'RsaSsaPssPublicKey', 'RsaSsaPkcs1PublicKey',):
    type_url = 'type.googleapis.com/google.crypto.tink.' + key_type_identifier
    key_manager = core.KeyManagerCcToPyWrapper(
        tink_bindings.PublicKeyVerifyKeyManager.from_cc_registry(type_url),
        _public_key_verify.PublicKeyVerify, _PublicKeyVerifyCcToPyWrapper)
    core.Registry.register_key_manager(key_manager, new_key_allowed=True)

  core.Registry.register_primitive_wrapper(
      _signature_wrapper.PublicKeySignWrapper())
  core.Registry.register_primitive_wrapper(
      _signature_wrapper.PublicKeyVerifyWrapper())



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.registry."""

from typing import cast, Mapping, Type, TypeVar
from absl.testing import absltest

from tink.proto import tink_pb2

from tink import aead
from tink import core
from tink import hybrid
from tink import mac
from tink import prf
from tink.mac import _mac_wrapper
from tink.prf import _prf_set_wrapper
from tink.testing import helper


P = TypeVar('P')


class DummyKeyManager(core.KeyManager[P]):

  def __init__(self, type_url: str, primitive_class: Type[P] = aead.Aead):
    self._type_url = type_url
    self._primitive_class = primitive_class

  def primitive_class(self) -> Type[P]:
    return self._primitive_class

  def primitive(self, key_data: tink_pb2.KeyData) -> P:
    return helper.FakeAead()

  def key_type(self) -> str:
    return self._type_url

  def new_key_data(self,
                   key_template: tink_pb2.KeyTemplate) -> tink_pb2.KeyData:
    return tink_pb2.KeyData(type_url=key_template.type_url)

  def does_support(self, type_url: str) -> bool:
    return self.key_type() == type_url


class UnsupportedKeyManager(DummyKeyManager[P]):

  def does_support(self, type_url: str) -> bool:
    return False


class DummyPrivateKeyManager(core.PrivateKeyManager[hybrid.HybridDecrypt]):

  def __init__(self, type_url: str):
    self._type_url = type_url

  def primitive_class(self) -> Type[hybrid.HybridDecrypt]:
    return hybrid.HybridDecrypt

  def primitive(self, key_data: tink_pb2.KeyData) -> hybrid.HybridDecrypt:
    return helper.FakeHybridDecrypt()

  def key_type(self) -> str:
    return self._type_url

  def new_key_data(self,
                   key_template: tink_pb2.KeyTemplate) -> tink_pb2.KeyData:
    return tink_pb2.KeyData()

  def public_key_data(
      self, private_key_data: tink_pb2.KeyData) -> tink_pb2.KeyData:
    return tink_pb2.KeyData(type_url='public_' + private_key_data.type_url)


class DummyMacWrapper(core.PrimitiveWrapper[mac.Mac, mac.Mac]):

  def wrap(self, pset: core.PrimitiveSet) -> mac.Mac:
    _ = pset
    return helper.FakeMac()

  def primitive_class(self) -> Type[mac.Mac]:
    return mac.Mac

  def input_primitive_class(self) -> Type[mac.Mac]:
    return mac.Mac


class InconsistentWrapper(core.PrimitiveWrapper[mac.Mac, mac.Mac]):

  def wrap(self, pset: core.PrimitiveSet) -> mac.Mac:
    _ = pset
    # returns a primitive of the wrong type
    return cast(mac.Mac, helper.FakeAead())

  def primitive_class(self) -> Type[mac.Mac]:
    return mac.Mac

  def input_primitive_class(self) -> Type[mac.Mac]:
    return mac.Mac


class _WrappedPrfSet(prf.PrfSet):
  """Implements PrfSet for a set of Prf primitives."""

  def __init__(self, primitive_set: core.PrimitiveSet):
    self._primitive_set = primitive_set

  def primary_id(self) -> int:
    return self._primitive_set.primary().key_id

  def all(self) -> Mapping[int, prf.Prf]:
    return {
        entry.key_id: entry.primitive
        for entry in self._primitive_set.raw_primitives()
    }

  def primary(self) -> prf.Prf:
    return self._primitive_set.primary().primitive


class PrfToPrfSetWrapper(core.PrimitiveWrapper[prf.Prf, prf.PrfSet]):
  """A PrimitiveWrapper that wraps Prfs into a PrfSet."""

  def wrap(self, primitives_set: core.PrimitiveSet) -> _WrappedPrfSet:
    return _WrappedPrfSet(primitives_set)

  def primitive_class(self) -> Type[prf.PrfSet]:
    return prf.PrfSet

  def input_primitive_class(self) -> Type[prf.Prf]:
    return prf.Prf


class _InvalidPrf(prf.Prf):

  def compute(self, input_data: bytes, output_length: int) -> bytes:
    raise core.TinkError('Invalid Prf')


class PrfToPrfWrapper(core.PrimitiveWrapper[prf.Prf, prf.Prf]):
  """A PrimitiveWrapper that wraps Prfs into a Prf."""

  def wrap(self, primitives_set: core.PrimitiveSet) -> prf.Prf:
    # This is needed because register_primitive_wrapper calls wrap with
    # an empty PrimitiveSet.
    if not primitives_set.raw_primitives():
      return _InvalidPrf()
    return primitives_set.primary().primitive

  def primitive_class(self) -> Type[prf.Prf]:
    return prf.Prf

  def input_primitive_class(self) -> Type[prf.Prf]:
    return prf.Prf


def _mac_set(mac_list) -> core.PrimitiveSet[mac.Mac]:
  """Converts a List of Mac in a PrimitiveSet and sets the last primary."""
  mac_set = core.new_primitive_set(mac.Mac)
  for i, primitive in enumerate(mac_list):
    mac_set.set_primary(
        mac_set.add_primitive(
            primitive,
            helper.fake_key(key_id=i, output_prefix_type=tink_pb2.RAW)))
  return mac_set


class RegistryTest(absltest.TestCase):

  def setUp(self):
    super().setUp()
    self.reg = core.Registry()
    self.reg.reset()

  def test_key_manager_no_exist(self):
    with self.assertRaises(core.TinkError):
      self.reg.key_manager('invalid')

  def test_key_manager_register(self):
    dummy_key_manager = DummyKeyManager('dummy_type_url')
    self.reg.register_key_manager(dummy_key_manager)
    self.assertEqual(dummy_key_manager, self.reg.key_manager('dummy_type_url'))

  def test_key_manager_reset(self):
    dummy_key_manager = DummyKeyManager('dummy_type_url')
    self.reg.register_key_manager(dummy_key_manager)
    self.reg.reset()
    with self.assertRaises(core.TinkError):
      self.reg.key_manager('dummy_type_url')

  def test_register_same_key_manager_twice(self):
    self.reg.register_key_manager(DummyKeyManager('dummy_type_url', aead.Aead))
    self.reg.register_key_manager(DummyKeyManager('dummy_type_url', aead.Aead))

  def test_register_unsupported_key_manager_fails(self):
    with self.assertRaises(core.TinkError):
      self.reg.register_key_manager(UnsupportedKeyManager('unsupported'))

  def test_key_manager_replace_fails(self):
    self.reg.register_key_manager(DummyKeyManager('dummy_type_url', aead.Aead))
    # Replacing the primitive_class for a type_url not allowed.
    with self.assertRaises(core.TinkError):
      self.reg.register_key_manager(
          DummyKeyManager('dummy_type_url', mac.Mac), new_key_allowed=False)

  def test_key_manager_disable_new_key_enable_fails(self):
    self.reg.register_key_manager(DummyKeyManager('dummy_type_url'))
    # Disable new keys.
    self.reg.register_key_manager(
        DummyKeyManager('dummy_type_url'), new_key_allowed=False)
    # Check new keys can't be enabled again.
    with self.assertRaises(core.TinkError):
      self.reg.register_key_manager(
          DummyKeyManager('dummy_type_url'), new_key_allowed=True)

  def test_primitive_ok(self):
    self.reg.register_key_manager(DummyKeyManager('dummy_type_url', aead.Aead))
    primitive = self.reg.primitive(
        tink_pb2.KeyData(type_url='dummy_type_url'), aead.Aead)
    self.assertIsInstance(primitive, helper.FakeAead)

  def test_primitive_fails_on_wrong_primitive(self):
    self.reg.register_key_manager(DummyKeyManager('dummy_type_url', aead.Aead))
    with self.assertRaises(core.TinkError):
      self.reg.primitive(tink_pb2.KeyData(type_url='dummy_type_url'), mac.Mac)

  def test_primitive_fails_on_subclass(self):
    self.reg.register_key_manager(
        DummyKeyManager('dummy_type_url', helper.FakeAead))
    with self.assertRaises(core.TinkError):
      self.reg.primitive(tink_pb2.KeyData(type_url='dummy_type_url'), aead.Aead)

  def test_new_key_data_success(self):
    self.reg.register_key_manager(DummyKeyManager('dummy_type_url'))
    key_template = tink_pb2.KeyTemplate(type_url='dummy_type_url')
    key_data = self.reg.new_key_data(key_template)
    self.assertEqual(key_data.type_url, 'dummy_type_url')

  def test_new_key_data_wrong_type_url(self):
    self.reg.register_key_manager(DummyKeyManager('dummy_type_url'))
    unknown_key_template = tink_pb2.KeyTemplate(type_url='unknown_type_url')
    with self.assertRaises(core.TinkError):
      self.reg.new_key_data(unknown_key_template)

  def test_new_key_data_no_new_key_allowed(self):
    self.reg.register_key_manager(
        DummyKeyManager('dummy_type_url'), new_key_allowed=False)
    key_template = tink_pb2.KeyTemplate(type_url='dummy_type_url')
    with self.assertRaises(core.TinkError):
      self.reg.new_key_data(key_template)

  def test_public_key_data_success(self):
    self.reg.register_key_manager(DummyPrivateKeyManager('dummy_type_url'))
    key_data = tink_pb2.KeyData(
        type_url='dummy_type_url',
        key_material_type=tink_pb2.KeyData.ASYMMETRIC_PRIVATE)
    public_key_data = self.reg.public_key_data(key_data)
    self.assertEqual(public_key_data.type_url, 'public_dummy_type_url')

  def test_public_key_data_fails_for_non_asymmetric_private_key(self):
    self.reg.register_key_manager(DummyPrivateKeyManager('dummy_type_url'))
    key_data = tink_pb2.KeyData(
        type_url='dummy_type_url',
        key_material_type=tink_pb2.KeyData.ASYMMETRIC_PUBLIC)
    with self.assertRaises(core.TinkError):
      self.reg.public_key_data(key_data)

  def test_public_key_data_fails_for_non_private_key_manager(self):
    self.reg.register_key_manager(DummyKeyManager('dummy_type_url'))
    key_data = tink_pb2.KeyData(
        type_url='dummy_type_url',
        key_material_type=tink_pb2.KeyData.ASYMMETRIC_PRIVATE)
    with self.assertRaises(core.TinkError):
      self.reg.public_key_data(key_data)

  def test_wrap_success(self):
    self.reg.register_primitive_wrapper(_mac_wrapper.MacWrapper())
    mac1 = helper.FakeMac('FakeMac1')
    mac2 = helper.FakeMac('FakeMac2')
    wrapped_mac = self.reg.wrap(_mac_set([mac1, mac2]), mac.Mac)
    wrapped_mac.verify_mac(mac1.compute_mac(b'data1'), b'data1')
    wrapped_mac.verify_mac(mac2.compute_mac(b'data2'), b'data2')
    wrapped_mac.verify_mac(wrapped_mac.compute_mac(b'data'), b'data')

  def test_wrap_with_primitive_class_success(self):
    self.reg.register_primitive_wrapper(_mac_wrapper.MacWrapper())
    mac1 = helper.FakeMac('FakeMac1')
    mac2 = helper.FakeMac('FakeMac2')
    wrapped_mac = self.reg.wrap(_mac_set([mac1, mac2]), mac.Mac)
    wrapped_mac.verify_mac(mac1.compute_mac(b'data1'), b'data1')
    wrapped_mac.verify_mac(mac2.compute_mac(b'data2'), b'data2')
    wrapped_mac.verify_mac(wrapped_mac.compute_mac(b'data'), b'data')

  def test_wrap_with_incompatible_primitive_class_fails(self):
    self.reg.register_primitive_wrapper(_mac_wrapper.MacWrapper())
    pset = core.new_primitive_set(prf.Prf)
    prf1 = helper.FakePrf('FakePrf1')
    pset.set_primary(
        pset.add_primitive(
            prf1,
            helper.fake_key(key_id=1234, output_prefix_type=tink_pb2.RAW)))
    with self.assertRaises(core.TinkError):
      _ = self.reg.wrap(pset, mac.Mac)

  def test_wrap_unknown_primitive(self):
    with self.assertRaises(core.TinkError):
      self.reg.wrap(_mac_set([helper.FakeMac()]), mac.Mac)

  def test_primitive_wrapper_reset(self):
    self.reg.register_primitive_wrapper(_mac_wrapper.MacWrapper())
    self.reg.reset()
    with self.assertRaises(core.TinkError):
      self.reg.wrap(_mac_set([helper.FakeMac()]), mac.Mac)

  def test_register_same_primitive_wrapper_twice(self):
    self.reg.register_primitive_wrapper(_mac_wrapper.MacWrapper())
    self.reg.register_primitive_wrapper(_mac_wrapper.MacWrapper())

  def test_register_different_primitive_wrappers_twice_fails(self):
    self.reg.register_primitive_wrapper(_mac_wrapper.MacWrapper())
    with self.assertRaises(core.TinkError):
      self.reg.register_primitive_wrapper(DummyMacWrapper())

  def test_register_inconsistent_wrapper_fails(self):
    with self.assertRaises(core.TinkError):
      self.reg.register_primitive_wrapper(InconsistentWrapper())

  def test_register_prf_to_prfset_wrapper_success(self):
    self.reg.register_primitive_wrapper(PrfToPrfSetWrapper())
    pset = core.new_primitive_set(prf.Prf)
    prf1 = helper.FakePrf('FakePrf1')
    prf2 = helper.FakePrf('FakePrf2')
    pset.add_primitive(
        prf1,
        helper.fake_key(key_id=1234, output_prefix_type=tink_pb2.RAW))
    pset.set_primary(
        pset.add_primitive(
            prf2,
            helper.fake_key(key_id=5678, output_prefix_type=tink_pb2.RAW)))
    wrapped_prf_set = self.reg.wrap(pset, prf.PrfSet)

    expected_output1 = prf1.compute(b'input', output_length=31)
    expected_output2 = prf2.compute(b'input', output_length=31)

    self.assertEqual(
        wrapped_prf_set.primary().compute(b'input', output_length=31),
        expected_output2)
    self.assertEqual(wrapped_prf_set.primary_id(), 5678)
    prfs = wrapped_prf_set.all()
    self.assertLen(prfs, 2)
    self.assertEqual(prfs[1234].compute(b'input', output_length=31),
                     expected_output1)
    self.assertEqual(prfs[5678].compute(b'input', output_length=31),
                     expected_output2)

  def test_two_wrappers_with_equal_primitive_class_fails(self):
    self.reg.register_primitive_wrapper(_prf_set_wrapper.PrfSetWrapper())
    # Only one wrapper for a output primitive is allowed.
    with self.assertRaises(core.TinkError):
      self.reg.register_primitive_wrapper(PrfToPrfSetWrapper())

  def test_two_wrappers_with_equal_input_primitive_class_success(self):
    self.reg.register_primitive_wrapper(PrfToPrfSetWrapper())
    # this is allowed, since PrfToPrfWrapper has a different output primitive.
    self.reg.register_primitive_wrapper(PrfToPrfWrapper())
    pset = core.new_primitive_set(prf.Prf)
    prf1 = helper.FakePrf('FakePrf1')
    prf2 = helper.FakePrf('FakePrf2')
    pset.add_primitive(
        prf1,
        helper.fake_key(key_id=1234, output_prefix_type=tink_pb2.RAW))
    pset.set_primary(
        pset.add_primitive(
            prf2,
            helper.fake_key(key_id=5678, output_prefix_type=tink_pb2.RAW)))

    wrapped_prf_set = self.reg.wrap(pset, prf.PrfSet)
    wrapped_prf = self.reg.wrap(pset, prf.Prf)

    expected_output2 = prf2.compute(b'input', output_length=31)

    self.assertEqual(
        wrapped_prf_set.primary().compute(b'input', output_length=31),
        expected_output2)
    self.assertEqual(
        wrapped_prf.compute(b'input', output_length=31),
        expected_output2)

if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.util.bytes_io."""

import io
import tempfile
from typing import BinaryIO, cast
from absl.testing import absltest
from absl.testing import parameterized

from tink.streaming_aead import _rewindable_input_stream
from tink.testing import bytes_io


class NonSeekableBytesIO(io.BytesIO):

  def seekable(self) -> bool:
    return False


def _rewindable(data,
                seekable) -> _rewindable_input_stream.RewindableInputStream:
  if seekable:
    b = cast(BinaryIO, io.BytesIO(data))
  else:
    b = cast(BinaryIO, NonSeekableBytesIO(data))
  return _rewindable_input_stream.RewindableInputStream(b)


class RewindableInputStreamTest(parameterized.TestCase):

  @parameterized.parameters([False, True])
  def test_read(self, seekable):
    with _rewindable(b'The quick brown fox', seekable) as f:
      self.assertEqual(b'The q', f.read(5))
      self.assertEqual(b'uick ', f.read(5))
      self.assertEqual(b'brown', f.read(5))
      self.assertEqual(b' fox', f.read(5))
      self.assertEqual(b'', f.read(5))
      self.assertEqual(b'', f.read(5))

  @parameterized.parameters([False, True])
  def test_read_no_argument(self, seekable):
    with _rewindable(b'The quick brown fox', seekable) as f:
      self.assertEqual(b'The quick brown fox', f.read())

  @parameterized.parameters([False, True])
  def test_read_minus_one(self, seekable):
    with _rewindable(b'The quick brown fox', seekable) as f:
      self.assertEqual(b'The quick brown fox', f.read(-1))

  @parameterized.parameters([False, True])
  def test_readall(self, seekable):
    with _rewindable(b'The quick brown fox', seekable) as f:
      self.assertEqual(b'The quick brown fox', f.readall())

  @parameterized.parameters([False, True])
  def test_rewind_read(self, seekable):
    with _rewindable(b'The quick brown fox', seekable) as f:
      self.assertEqual(b'The quick', f.read(9))
      f.rewind()
      self.assertEqual(b'The ', f.read(4))
      # this only reads the rest of current buffer content.
      self.assertEqual(b'quick', f.read(100))
      self.assertEqual(b' brown fox', f.read())

  @parameterized.parameters([False, True])
  def test_rewind_readall(self, seekable):
    with _rewindable(b'The quick brown fox', seekable) as f:
      self.assertEqual(b'The q', f.read(5))
      f.rewind()
      # this must read the whole file.
      self.assertEqual(b'The quick brown fox', f.read())

  @parameterized.parameters([False, True])
  def test_rewind_twice(self, seekable):
    with _rewindable(b'The quick brown fox', seekable) as f:
      self.assertEqual(b'The q', f.read(5))
      f.rewind()
      self.assertEqual(b'The q', f.read(5))
      self.assertEqual(b'uick ', f.read(5))
      f.rewind()
      self.assertEqual(b'The quick brown fox', f.read())

  @parameterized.parameters([False, True])
  def test_disable_rewind(self, seekable):
    with _rewindable(b'The quick brown fox', seekable) as f:
      self.assertEqual(b'The q', f.read(5))
      f.rewind()
      f.disable_rewind()
      # this only reads the current buffer content.
      self.assertEqual(b'The q', f.read(100))
      self.assertEqual(b'u', f.read(1))
      self.assertEmpty(f._buffer)
      self.assertEqual(b'ick brown fox', f.read())

  @parameterized.parameters([False, True])
  def test_disable_rewind_readall(self, seekable):
    with _rewindable(b'The quick brown fox', seekable) as f:
      self.assertEqual(b'The q', f.read(5))
      f.rewind()
      f.disable_rewind()
      self.assertEqual(b'The quick brown fox', f.read())

  def test_nonreadable_input_fail(self):
    with tempfile.TemporaryFile('wb') as f:
      with self.assertRaises(ValueError):
        _ = _rewindable_input_stream.RewindableInputStream(cast(BinaryIO, f))


class RewindableInputStreamSlowTest(parameterized.TestCase):
  """Tests "slow" input streams where read returns None or BlockingIOError.

  Normally, this should not happen in blocking streams.
  """

  @parameterized.parameters([False, True])
  def test_read_slow(self, seekable):
    input_stream = bytes_io.SlowBytesIO(b'The quick brown fox', seekable)
    with _rewindable_input_stream.RewindableInputStream(
        cast(BinaryIO, input_stream)) as f:
      self.assertIsNone(f.read(10))
      self.assertEqual(b'The q', f.read(10))
      self.assertEqual(b'uick ', f.read(10))
      self.assertIsNone(f.read(10))
      self.assertEqual(b'brown', f.read(10))
      self.assertEqual(b' fox', f.read(10))
      self.assertIsNone(f.read(10))
      self.assertEqual(b'', f.read(10))

  @parameterized.parameters([False, True])
  def test_read_slow_raw(self, seekable):
    input_stream = bytes_io.SlowReadableRawBytes(b'The quick brown fox',
                                                 seekable)
    with _rewindable_input_stream.RewindableInputStream(
        cast(BinaryIO, input_stream)) as f:
      self.assertIsNone(f.read(10))
      self.assertEqual(b'The q', f.read(10))
      self.assertEqual(b'uick ', f.read(10))
      self.assertIsNone(f.read(10))
      self.assertEqual(b'brown', f.read(10))
      self.assertEqual(b' fox', f.read(10))
      self.assertIsNone(f.read(10))
      self.assertEqual(b'', f.read(10))

  @parameterized.parameters([False, True])
  def test_read_slow_raw_readall(self, seekable):
    input_stream = bytes_io.SlowReadableRawBytes(b'The quick brown fox',
                                                 seekable)
    with _rewindable_input_stream.RewindableInputStream(
        cast(BinaryIO, input_stream)) as f:
      self.assertIsNone(f.readall())
      self.assertEqual(b'The quick ', f.readall())
      self.assertEqual(b'brown fox', f.readall())
      self.assertEqual(b'', f.readall())


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink._mac_key_manager."""

from absl.testing import absltest
from absl.testing import parameterized

from tink.proto import common_pb2
from tink.proto import hmac_pb2

import tink
from tink import core
from tink import mac


def setUpModule():
  mac.register()


class MacKeyManagerTest(parameterized.TestCase):

  def test_new_key_data_hmac(self):
    key_template = mac.mac_key_templates.create_hmac_key_template(
        key_size=16, tag_size=24, hash_type=common_pb2.SHA256)
    key_manager = core.Registry.key_manager(key_template.type_url)
    key_data = key_manager.new_key_data(key_template)
    self.assertEqual(key_data.type_url, key_manager.key_type())
    key = hmac_pb2.HmacKey()
    key.ParseFromString(key_data.value)
    self.assertEqual(key.version, 0)
    self.assertEqual(key.params.hash, common_pb2.SHA256)
    self.assertEqual(key.params.tag_size, 24)
    self.assertLen(key.key_value, 16)

  def test_invalid_params_throw_exception(self):
    key_template = mac.mac_key_templates.create_hmac_key_template(
        key_size=16, tag_size=9, hash_type=common_pb2.SHA256)
    with self.assertRaises(core.TinkError):
      tink.new_keyset_handle(key_template)

  @parameterized.parameters([
      mac.mac_key_templates.HMAC_SHA256_128BITTAG,
      mac.mac_key_templates.HMAC_SHA256_256BITTAG,
      mac.mac_key_templates.HMAC_SHA512_256BITTAG,
      mac.mac_key_templates.HMAC_SHA512_512BITTAG,
  ])
  def test_mac_success(self, key_template):
    keyset_handle = tink.new_keyset_handle(key_template)
    mac_primitive = keyset_handle.primitive(mac.Mac)
    data = b'data'
    tag = mac_primitive.compute_mac(data)
    self.assertGreaterEqual(len(tag), 16)
    # No exception raised, no return value.
    self.assertIsNone(mac_primitive.verify_mac(tag, data))

  @parameterized.parameters([
      mac.mac_key_templates.HMAC_SHA256_128BITTAG,
      mac.mac_key_templates.HMAC_SHA256_256BITTAG,
      mac.mac_key_templates.HMAC_SHA512_256BITTAG,
      mac.mac_key_templates.HMAC_SHA512_512BITTAG,
  ])
  def test_mac_wrong(self, key_template):
    keyset_handle = tink.new_keyset_handle(key_template)
    mac_primitive = keyset_handle.primitive(mac.Mac)
    with self.assertRaises(core.TinkError):
      mac_primitive.verify_mac(b'0123456789ABCDEF', b'data')


if __name__ == '__main__':
  absltest.main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/ecdsa.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/ecdsa.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tink.proto import common_pb2 as tink_dot_proto_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16tink/proto/ecdsa.proto\x12\x12google.crypto.tink\x1a\x17tink/proto/common.proto\"\xb2\x01\n\x0b\x45\x63\x64saParams\x12/\n\thash_type\x18\x01 \x01(\x0e\x32\x1c.google.crypto.tink.HashType\x12\x34\n\x05\x63urve\x18\x02 \x01(\x0e\x32%.google.crypto.tink.EllipticCurveType\x12<\n\x08\x65ncoding\x18\x03 \x01(\x0e\x32*.google.crypto.tink.EcdsaSignatureEncoding\"h\n\x0e\x45\x63\x64saPublicKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12/\n\x06params\x18\x02 \x01(\x0b\x32\x1f.google.crypto.tink.EcdsaParams\x12\t\n\x01x\x18\x03 \x01(\x0c\x12\t\n\x01y\x18\x04 \x01(\x0c\"m\n\x0f\x45\x63\x64saPrivateKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x36\n\npublic_key\x18\x02 \x01(\x0b\x32\".google.crypto.tink.EcdsaPublicKey\x12\x11\n\tkey_value\x18\x03 \x01(\x0c\"R\n\x0e\x45\x63\x64saKeyFormat\x12/\n\x06params\x18\x02 \x01(\x0b\x32\x1f.google.crypto.tink.EcdsaParams\x12\x0f\n\x07version\x18\x03 \x01(\r*G\n\x16\x45\x63\x64saSignatureEncoding\x12\x14\n\x10UNKNOWN_ENCODING\x10\x00\x12\x0e\n\nIEEE_P1363\x10\x01\x12\x07\n\x03\x44\x45R\x10\x02\x42X\n\x1c\x63om.google.crypto.tink.protoP\x01Z6github.com/tink-crypto/tink-go/v2/proto/ecdsa_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.ecdsa_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z6github.com/tink-crypto/tink-go/v2/proto/ecdsa_go_proto'
  _globals['_ECDSASIGNATUREENCODING']._serialized_start=553
  _globals['_ECDSASIGNATUREENCODING']._serialized_end=624
  _globals['_ECDSAPARAMS']._serialized_start=72
  _globals['_ECDSAPARAMS']._serialized_end=250
  _globals['_ECDSAPUBLICKEY']._serialized_start=252
  _globals['_ECDSAPUBLICKEY']._serialized_end=356
  _globals['_ECDSAPRIVATEKEY']._serialized_start=358
  _globals['_ECDSAPRIVATEKEY']._serialized_end=467
  _globals['_ECDSAKEYFORMAT']._serialized_start=469
  _globals['_ECDSAKEYFORMAT']._serialized_end=551
# @@protoc_insertion_point(module_scope)



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This module defines the interface for AEAD."""

import abc


class Aead(metaclass=abc.ABCMeta):
  """The interface for authenticated encryption with associated data.

  Implementations of this interface are secure against adaptive
  chosen ciphertext attacks.  Encryption with associated data ensures
  authenticity and integrity of that data, but not its secrecy.
  (see RFC 5116, https://tools.ietf.org/html/rfc5116)
  """

  @abc.abstractmethod
  def encrypt(self, plaintext: bytes, associated_data: bytes) -> bytes:
    """Encrypts plaintext with associated_data.

    The ciphertext allows for checking authenticity and integrity of the
    associated data, but does not guarantee its secrecy.

    Args:
      plaintext: bytes. The data to be encrypted.
      associated_data: bytes. The associated data, that will be authenticated.
    Returns:
      the resulting ciphertext as bytes.
    Raises:
      tink.TinkError if the encryption fails.
    """
    raise NotImplementedError()

  @abc.abstractmethod
  def decrypt(self, ciphertext: bytes, associated_data: bytes) -> bytes:
    """Decrypts ciphertext with associated_data.

    The decryption verifies the authenticity and integrity of the associated
    data, but there are no guarantees with respect to secrecy of that data.

    Args:
      ciphertext: bytes. The data to be decrypted.
      associated_data: bytes. The associated data.
    Returns:
      the resulting plaintext as bytes.
    Raises:
      tink.TinkError if the decryption fails.
    """
    raise NotImplementedError()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Python primitive set wrapper for the JwtMac primitive."""

from typing import Type

from tink.proto import tink_pb2
from tink import core
from tink.jwt import _jwt_error
from tink.jwt import _jwt_format
from tink.jwt import _jwt_mac
from tink.jwt import _jwt_validator
from tink.jwt import _raw_jwt
from tink.jwt import _verified_jwt


class _WrappedJwtMac(_jwt_mac.JwtMac):
  """A wrapped JwtMac."""

  def __init__(self, pset: core.PrimitiveSet):
    self._primitive_set = pset

  def compute_mac_and_encode(self, raw_jwt: _raw_jwt.RawJwt) -> str:
    """Computes a MAC and encodes the token.

    Args:
      raw_jwt: The RawJwt token to be MACed and encoded.

    Returns:
      The MACed token encoded in the JWS compact serialization format.
    Raises:
      tink.TinkError if the operation fails.
    """
    primary = self._primitive_set.primary()
    kid = _jwt_format.get_kid(primary.key_id, primary.output_prefix_type)
    return primary.primitive.compute_mac_and_encode_with_kid(raw_jwt, kid)

  def verify_mac_and_decode(
      self, compact: str,
      validator: _jwt_validator.JwtValidator) -> _verified_jwt.VerifiedJwt:
    """Verifies, validates and decodes a MACed compact JWT token.

    Args:
      compact: A MACed token encoded in the JWS compact serialization format.
      validator: A JwtValidator that validates the token.

    Returns:
      A VerifiedJwt.
    Raises:
      tink.TinkError if the operation fails.
    """
    interesting_error = None
    for entries in self._primitive_set.all():
      for entry in entries:
        try:
          kid = _jwt_format.get_kid(entry.key_id, entry.output_prefix_type)
          return entry.primitive.verify_mac_and_decode_with_kid(
              compact, validator, kid)
        except core.TinkError as e:
          if isinstance(e, _jwt_error.JwtInvalidError):
            interesting_error = e
          pass
    if interesting_error:
      raise interesting_error
    raise core.TinkError('invalid MAC')


def _validate_primitive_set(pset: core.PrimitiveSet):
  for entries in pset.all():
    for entry in entries:
      if (entry.output_prefix_type != tink_pb2.RAW and
          entry.output_prefix_type != tink_pb2.TINK):
        raise core.TinkError('unsupported OutputPrefixType')


class _Wrapper(core.PrimitiveWrapper[_jwt_mac.JwtMacInternal, _jwt_mac.JwtMac]):
  """A wrapper for JwtMac."""

  def wrap(self, pset: core.PrimitiveSet) -> _jwt_mac.JwtMac:
    _validate_primitive_set(pset)
    return _WrappedJwtMac(pset)

  def primitive_class(self) -> Type[_jwt_mac.JwtMac]:
    return _jwt_mac.JwtMac

  def input_primitive_class(self) -> Type[_jwt_mac.JwtMacInternal]:
    return _jwt_mac.JwtMacInternal


def register():
  core.Registry.register_primitive_wrapper(_Wrapper())



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Writes Keysets to file."""

import abc
from typing import BinaryIO, TextIO

from google.protobuf import json_format
from tink.proto import tink_pb2
from tink import core


class KeysetWriter(metaclass=abc.ABCMeta):
  """Knows how to write keysets to some storage system."""

  @abc.abstractmethod
  def write(self, keyset: tink_pb2.Keyset) -> None:
    """Tries to write a tink_pb2.Keyset to some storage system."""
    raise NotImplementedError()

  @abc.abstractmethod
  def write_encrypted(self, encrypted_keyset: tink_pb2.EncryptedKeyset) -> None:
    """Tries to write an tink_pb2.EncryptedKeyset to some storage system."""
    raise NotImplementedError()


# Deprecated. Instead, use the serialize functions in
# tink.json_proto_keyset_format.
class JsonKeysetWriter(KeysetWriter):
  """Writes keysets in proto JSON wire format to some storage system.

  cf. https://developers.google.com/protocol-buffers/docs/encoding
  """

  def __init__(self, text_io_stream: TextIO):
    self._io_stream = text_io_stream

  def write(self, keyset: tink_pb2.Keyset) -> None:
    if not isinstance(keyset, tink_pb2.Keyset):
      raise core.TinkError('invalid keyset.')
    json_keyset = json_format.MessageToJson(keyset)
    self._io_stream.write(json_keyset)
    self._io_stream.flush()

  def write_encrypted(self, encrypted_keyset: tink_pb2.EncryptedKeyset) -> None:
    if not isinstance(encrypted_keyset, tink_pb2.EncryptedKeyset):
      raise core.TinkError('invalid encrypted keyset.')
    json_keyset = json_format.MessageToJson(encrypted_keyset)
    self._io_stream.write(json_keyset)
    self._io_stream.flush()


# Deprecated. Instead, use the serialize functions in tink.proto_keyset_format.
class BinaryKeysetWriter(KeysetWriter):
  """Writes keysets in proto binary wire format to some storage system.

  cf. https://developers.google.com/protocol-buffers/docs/encoding
  """

  def __init__(self, binary_io_stream: BinaryIO):
    self._io_stream = binary_io_stream

  def write(self, keyset: tink_pb2.Keyset) -> None:
    if not isinstance(keyset, tink_pb2.Keyset):
      raise core.TinkError('invalid keyset.')
    self._io_stream.write(keyset.SerializeToString())
    self._io_stream.flush()

  def write_encrypted(self, encrypted_keyset: tink_pb2.EncryptedKeyset) -> None:
    if not isinstance(encrypted_keyset, tink_pb2.EncryptedKeyset):
      raise core.TinkError('invalid encrypted keyset.')
    encrypted_keyset_without_keyset_info = tink_pb2.EncryptedKeyset(
        encrypted_keyset=encrypted_keyset.encrypted_keyset
    )
    self._io_stream.write(
        encrypted_keyset_without_keyset_info.SerializeToString()
    )
    self._io_stream.flush()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""PrfSet wrapper."""

from typing import Type, Mapping
from tink.proto import tink_pb2
from tink import core
from tink.prf import _prf_set


class _WrappedPrfSet(_prf_set.PrfSet):
  """Implements PrfSet for a set of PrfSet primitives."""

  def __init__(self, primitive_set: core.PrimitiveSet):
    self._primitive_set = primitive_set

  def primary_id(self) -> int:
    return self._primitive_set.primary().key_id

  def all(self) -> Mapping[int, _prf_set.Prf]:
    return {
        entry.key_id: entry.primitive
        for entry in self._primitive_set.raw_primitives()
    }

  def primary(self) -> _prf_set.Prf:
    return self._primitive_set.primary().primitive


def _validate_primitive_set(pset: core.PrimitiveSet):
  for entries in pset.all():
    for entry in entries:
      if entry.output_prefix_type != tink_pb2.RAW:
        raise core.TinkError('unsupported OutputPrefixType')


class PrfSetWrapper(core.PrimitiveWrapper[_prf_set.Prf, _prf_set.PrfSet]):
  """A PrimitiveWrapper for the PrfSet primitive.

  The returned primitive works with a keyset (rather than a single key). To sign
  a message, it uses the primary key in the keyset, and prepends to the
  signature a certain prefix associated with the primary key.
  """

  def wrap(self, primitives_set: core.PrimitiveSet) -> _WrappedPrfSet:
    _validate_primitive_set(primitives_set)
    return _WrappedPrfSet(primitives_set)

  def primitive_class(self) -> Type[_prf_set.PrfSet]:
    return _prf_set.PrfSet

  def input_primitive_class(self) -> Type[_prf_set.Prf]:
    return _prf_set.Prf



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/ml_dsa.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/ml_dsa.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17tink/proto/ml_dsa.proto\x12\x12google.crypto.tink\"I\n\x0bMlDsaParams\x12:\n\x0fml_dsa_instance\x18\x01 \x01(\x0e\x32!.google.crypto.tink.MlDsaInstance\"R\n\x0eMlDsaKeyFormat\x12\x0f\n\x07version\x18\x01 \x01(\r\x12/\n\x06params\x18\x02 \x01(\x0b\x32\x1f.google.crypto.tink.MlDsaParams\"e\n\x0eMlDsaPublicKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x11\n\tkey_value\x18\x02 \x01(\x0c\x12/\n\x06params\x18\x03 \x01(\x0b\x32\x1f.google.crypto.tink.MlDsaParams\"m\n\x0fMlDsaPrivateKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x11\n\tkey_value\x18\x02 \x01(\x0c\x12\x36\n\npublic_key\x18\x03 \x01(\x0b\x32\".google.crypto.tink.MlDsaPublicKey*J\n\rMlDsaInstance\x12\x1b\n\x17ML_DSA_UNKNOWN_INSTANCE\x10\x00\x12\r\n\tML_DSA_65\x10\x01\x12\r\n\tML_DSA_87\x10\x02\x42V\n\x1c\x63om.google.crypto.tink.protoP\x01Z4github.com/tink-crypto/tink-go/v2/proto/ml_dsa_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.ml_dsa_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z4github.com/tink-crypto/tink-go/v2/proto/ml_dsa_proto'
  _globals['_MLDSAINSTANCE']._serialized_start=420
  _globals['_MLDSAINSTANCE']._serialized_end=494
  _globals['_MLDSAPARAMS']._serialized_start=47
  _globals['_MLDSAPARAMS']._serialized_end=120
  _globals['_MLDSAKEYFORMAT']._serialized_start=122
  _globals['_MLDSAKEYFORMAT']._serialized_end=204
  _globals['_MLDSAPUBLICKEY']._serialized_start=206
  _globals['_MLDSAPUBLICKEY']._serialized_end=307
  _globals['_MLDSAPRIVATEKEY']._serialized_start=309
  _globals['_MLDSAPRIVATEKEY']._serialized_end=418
# @@protoc_insertion_point(module_scope)



# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for _hcvault_kms_aead.py."""

import base64
import http.server
import json
import re
import threading
from absl.testing import absltest
from absl.testing import parameterized
import hvac
import tink
from tink import aead
from tink.integration import hcvault

_TOKEN = ''
_KEY_PATH = 'transit/keys/key-1'
_PORT = 8205
_VAULT_URI = f'http://localhost:{_PORT}'


_KEY_NAME_TO_AEAD = {}


def _new_aead() -> aead.Aead:
  handle = tink.new_keyset_handle(aead.aead_key_templates.AES128_GCM)
  return handle.primitive(aead.Aead)


def setUpModule():
  global _KEY_NAME_TO_AEAD
  aead.register()
  _KEY_NAME_TO_AEAD = {k: _new_aead() for k in ['key-1', 'key-2']}


class MockHandler(http.server.BaseHTTPRequestHandler):

  def do_POST(self):  # pylint: disable=invalid-name
    path_prefix, key_name = self.path.rsplit('/', 1)
    if key_name not in _KEY_NAME_TO_AEAD:
      self.send_error(404)
      return
    if path_prefix == '/v1/transit/encrypt':
      post_body = self.rfile.read(int(self.headers.get('Content-Length')))
      req = json.loads(post_body)
      ret = self.encrypt(key_name, req)
      self.send_response(200)
      self.send_header('Content-Type', 'application/json')
      self.end_headers()
      self.wfile.write(ret.encode('utf-8'))
      return

    if path_prefix == '/v1/transit/decrypt':
      try:
        post_body = self.rfile.read(int(self.headers.get('Content-Length')))
        req = json.loads(post_body)
        ret = self.decrypt(key_name, req)
        self.send_response(200)
        self.send_header('Content-Type', 'application/json')
        self.end_headers()
        self.wfile.write(ret.encode('utf-8'))
      except (OSError, BlockingIOError):
        self.send_error(400)
      return

    self.send_error(404)

  def do_GET(self):  # pylint: disable=invalid-name
    self.send_error(404)

  def do_PUT(self):  # pylint: disable=invalid-name
    self.send_error(404)

  def encrypt(self, key_name, data):
    plaintext = base64.b64decode(data['plaintext'])
    context = base64.b64decode(data['context'])
    ciphertext = _KEY_NAME_TO_AEAD[key_name].encrypt(plaintext, context)
    resp = {'ciphertext': base64.b64encode(ciphertext).decode()}
    return json.dumps({'data': resp})

  def decrypt(self, key_name, data):
    ciphertext = base64.b64decode(data['ciphertext'])
    context = base64.b64decode(data['context'])
    plaintext = _KEY_NAME_TO_AEAD[key_name].decrypt(ciphertext, context)
    resp = {'plaintext': base64.b64encode(plaintext).decode()}
    return json.dumps({'data': resp})


class HcVaultKmsAeadTest(parameterized.TestCase):

  def setUp(self):
    super().setUp()
    self.server = http.server.HTTPServer(('localhost', _PORT), MockHandler)
    threading.Thread(target=self.server.serve_forever).start()

  def tearDown(self):
    super().tearDown()
    if self.server:
      self.server.shutdown()
      self.server.server_close()

  def test_encrypt_decrypt_with_empty_associated_data(self):
    client = hvac.Client(url=_VAULT_URI, token=_TOKEN, verify=False)
    vaultaead = hcvault.new_aead(_KEY_PATH, client)
    plaintext = bytes(i for i in range(256))
    ciphertext = vaultaead.encrypt(plaintext, associated_data=b'')
    self.assertEqual(
        plaintext, vaultaead.decrypt(ciphertext, associated_data=b'')
    )

  def test_encrypt_decrypt_fails_with_nonempty_associated_data(self):
    client = hvac.Client(url=_VAULT_URI, token=_TOKEN, verify=False)
    vaultaead = hcvault.new_aead(_KEY_PATH, client)
    expected_error_msg_re = re.compile(r'.*only allows empty associated data.*')
    with self.assertRaisesRegex(tink.TinkError, expected_error_msg_re):
      _ = vaultaead.encrypt(plaintext=b'hello', associated_data=b'non-empty')
    with self.assertRaisesRegex(tink.TinkError, expected_error_msg_re):
      _ = vaultaead.decrypt(ciphertext=b'hello', associated_data=b'non-empty')

  def test_encrypt_fails_with_unknown_key_name(self):
    client = hvac.Client(url=_VAULT_URI, token=_TOKEN, verify=False)
    vaultaead = hcvault.new_aead('transit/keys/unknown-key', client)
    with self.assertRaises(tink.TinkError):
      _ = vaultaead.encrypt(plaintext=b'plaintext', associated_data=b'')

  def test_decrypt_with_wrong_key_fails(self):
    client = hvac.Client(url=_VAULT_URI, token=_TOKEN, verify=False)
    vaultaead1 = hcvault.new_aead('transit/keys/key-1', client)
    vaultaead2 = hcvault.new_aead('transit/keys/key-2', client)
    plaintext = b'plaintext'
    associated_data = b''
    ciphertext1 = vaultaead1.encrypt(plaintext, associated_data)
    with self.assertRaises(tink.TinkError):
      _ = vaultaead2.decrypt(ciphertext1, associated_data)

  @parameterized.named_parameters([
      ('simple', '/transit/keys/key-1', 'transit', 'key-1'),
      ('simple_no_leading_slash', 'transit/keys/key-1', 'transit', 'key-1'),
      (
          'escaped',
          '/transit/keys/this%2Band+that',
          'transit',
          'this%252Band%2Bthat',
      ),
      (
          'sub_path',
          '/teams/billing/something/transit/keys/pci-key',
          'teams/billing/something/transit',
          'pci-key',
      ),
      (
          'transit_twice',
          '/transit/keys/something/transit/keys/my-key',
          'transit/keys/something/transit',
          'my-key',
      ),
      ('mount_not_named_transit', '/cipher/keys/hi', 'cipher', 'hi'),
  ])
  def test_valid_get_endpoint_paths(
      self, path, expected_mount, expected_key_name
  ):
    mount, path = hcvault._hcvault_kms_aead._get_endpoint_paths(path)
    self.assertEqual(mount, expected_mount)
    self.assertEqual(path, expected_key_name)

  @parameterized.named_parameters([
      ('empty', ''),
      ('slash_only', '/'),
      ('no_mount', '/keys/foo'),
      ('traling_slash', 'mount/keys/foo/'),
      ('invalid_mount', '////keys/foo'),
      ('invalid_mount_and_trailing_slash', '////keys/foo/'),
      ('invalid_mount_with_empty_component_in_between', '/foo//bar/keys/baz'),
      ('invalid_mount_with_empty_trailing_components', '/foo/bar///keys/baz'),
      ('invalid_key_name', '/transit/keys/bar/baz'),
  ])
  def test_invalid_path_raises_error(self, path):
    client = hvac.Client(url=_VAULT_URI, token=_TOKEN, verify=False)
    with self.assertRaises(tink.TinkError):
      hcvault.new_aead(path, client)


if __name__ == '__main__':
  absltest.main()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.cleartext_keyset_handle."""

import io

from absl.testing import absltest

import tink
from tink import cleartext_keyset_handle
from tink import core
from tink import mac
from tink import tink_config


def setUpModule():
  tink_config.register()


class CleartextKeysetHandleTest(absltest.TestCase):

  def test_write_read(self):
    handle = tink.new_keyset_handle(
        mac.mac_key_templates.HMAC_SHA256_128BITTAG)
    output_stream = io.BytesIO()
    writer = tink.BinaryKeysetWriter(output_stream)
    cleartext_keyset_handle.write(writer, handle)
    reader = tink.BinaryKeysetReader(output_stream.getvalue())
    handle2 = cleartext_keyset_handle.read(reader)
    # Check that handle2 has the same primitive as handle.
    handle2.primitive(mac.Mac).verify_mac(
        handle.primitive(mac.Mac).compute_mac(b'data'), b'data')

  def test_from_keyset(self):
    handle = tink.new_keyset_handle(
        mac.mac_key_templates.HMAC_SHA256_128BITTAG)
    keyset = handle._keyset
    handle2 = cleartext_keyset_handle.from_keyset(keyset)
    # Check that handle2 has the same primitive as handle.
    handle2.primitive(mac.Mac).verify_mac(
        handle.primitive(mac.Mac).compute_mac(b'data'), b'data')

  def test_read_empty_keyset_fails(self):
    with self.assertRaises(core.TinkError):
      cleartext_keyset_handle.read(tink.BinaryKeysetReader(b''))

if __name__ == '__main__':
  absltest.main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/x_aes_gcm.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/x_aes_gcm.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1atink/proto/x_aes_gcm.proto\x12\x12google.crypto.tink\"\"\n\rXAesGcmParams\x12\x11\n\tsalt_size\x18\x01 \x01(\r\"\\\n\x10XAesGcmKeyFormat\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x31\n\x06params\x18\x03 \x01(\x0b\x32!.google.crypto.tink.XAesGcmParamsJ\x04\x08\x02\x10\x03\"c\n\nXAesGcmKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x31\n\x06params\x18\x02 \x01(\x0b\x32!.google.crypto.tink.XAesGcmParams\x12\x11\n\tkey_value\x18\x03 \x01(\x0c\x42\\\n\x1c\x63om.google.crypto.tink.protoP\x01Z:github.com/tink-crypto/tink-go/v2/proto/x_aes_gcm_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.x_aes_gcm_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z:github.com/tink-crypto/tink-go/v2/proto/x_aes_gcm_go_proto'
  _globals['_XAESGCMPARAMS']._serialized_start=50
  _globals['_XAESGCMPARAMS']._serialized_end=84
  _globals['_XAESGCMKEYFORMAT']._serialized_start=86
  _globals['_XAESGCMKEYFORMAT']._serialized_end=178
  _globals['_XAESGCMKEY']._serialized_start=180
  _globals['_XAESGCMKEY']._serialized_end=279
# @@protoc_insertion_point(module_scope)



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Python wrapper of the wrapped C++ Deterministic AEAD key manager."""

from tink import core
from tink.cc.pybind import tink_bindings
from tink.daead import _deterministic_aead
from tink.daead import _deterministic_aead_wrapper


class _DeterministicAeadCcToPyWrapper(_deterministic_aead.DeterministicAead):
  """Transforms C++ DeterministicAead into a Python primitive."""

  def __init__(self, cc_deterministic_aead):
    self._deterministic_aead = cc_deterministic_aead

  @core.use_tink_errors
  def encrypt_deterministically(self, plaintext: bytes,
                                associated_data: bytes) -> bytes:
    return self._deterministic_aead.encrypt_deterministically(
        plaintext, associated_data)

  @core.use_tink_errors
  def decrypt_deterministically(self, ciphertext: bytes,
                                associated_data: bytes) -> bytes:
    return self._deterministic_aead.decrypt_deterministically(
        ciphertext, associated_data)


def register():
  """Registers all deterministic AEAD key managers and the wrapper in the Python Registry."""
  tink_bindings.register()

  type_url = 'type.googleapis.com/google.crypto.tink.AesSivKey'
  key_manager = core.KeyManagerCcToPyWrapper(
      tink_bindings.DeterministicAeadKeyManager.from_cc_registry(type_url),
      _deterministic_aead.DeterministicAead, _DeterministicAeadCcToPyWrapper)
  core.Registry.register_key_manager(key_manager, new_key_allowed=True)
  core.Registry.register_primitive_wrapper(
      _deterministic_aead_wrapper.DeterministicAeadWrapper())





# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Pre-generated KeyTemplate for PublicKeySign and PublicKeyVerify.

One can use these templates to generate a new tink_pb2.Keyset with
tink_pb2.KeysetHandle. To generate a new keyset that contains a single
EcdsaPrivateKey, one can do:

handle = keyset_handle.KeysetHandle(signature_key_templates.ECDSA_P256);
"""

import warnings

from tink.proto import common_pb2
from tink.proto import ecdsa_pb2
from tink.proto import rsa_ssa_pkcs1_pb2
from tink.proto import rsa_ssa_pss_pb2
from tink.proto import tink_pb2
from tink.internal import big_integer_util


_prefix = 'type.googleapis.com/google.crypto.tink.'
_ECDSA_KEY_TYPE_URL = _prefix + 'EcdsaPrivateKey'
_ED25519_KEY_TYPE_URL = _prefix + 'Ed25519PrivateKey'
_RSA_PKCS1_KEY_TYPE_URL = _prefix + 'RsaSsaPkcs1PrivateKey'
_RSA_PSS_KEY_TYPE_URL = _prefix + 'RsaSsaPssPrivateKey'
_RSA_F4 = 65537


def _create_ecdsa_key_template(
    hash_type: common_pb2.HashType,
    curve: common_pb2.EllipticCurveType,
    encoding: ecdsa_pb2.EcdsaSignatureEncoding,
    output_prefix_type: tink_pb2.OutputPrefixType = tink_pb2.TINK
) -> tink_pb2.KeyTemplate:
  """Creates a KeyTemplate containing an EcdsaKeyFormat."""
  params = ecdsa_pb2.EcdsaParams(
      hash_type=hash_type, curve=curve, encoding=encoding)
  key_format = ecdsa_pb2.EcdsaKeyFormat(params=params)
  key_template = tink_pb2.KeyTemplate(
      value=key_format.SerializeToString(),
      type_url=_ECDSA_KEY_TYPE_URL,
      output_prefix_type=output_prefix_type)

  return key_template


def _create_rsa_ssa_pkcs1_key_template(
    hash_type: common_pb2.HashType, modulus_size: int,
    public_exponent: int) -> tink_pb2.KeyTemplate:
  """Creates a KeyTemplate containing an RsaSsaPkcs1KeyFormat."""
  params = rsa_ssa_pkcs1_pb2.RsaSsaPkcs1Params(hash_type=hash_type)
  key_format = rsa_ssa_pkcs1_pb2.RsaSsaPkcs1KeyFormat(
      params=params,
      modulus_size_in_bits=modulus_size,
      public_exponent=big_integer_util.num_to_bytes(public_exponent))
  key_template = tink_pb2.KeyTemplate(
      value=key_format.SerializeToString(),
      type_url=_RSA_PKCS1_KEY_TYPE_URL,
      output_prefix_type=tink_pb2.TINK)

  return key_template


def _create_rsa_ssa_pss_key_template(
    sig_hash: common_pb2.HashType, mgf1_hash: common_pb2.HashType,
    salt_length: int, modulus_size: int,
    public_exponent: int) -> tink_pb2.KeyTemplate:
  """Creates a KeyTemplate containing an RsaSsaPssKeyFormat."""
  params = rsa_ssa_pss_pb2.RsaSsaPssParams(
      sig_hash=sig_hash, mgf1_hash=mgf1_hash, salt_length=salt_length)
  key_format = rsa_ssa_pss_pb2.RsaSsaPssKeyFormat(
      params=params,
      modulus_size_in_bits=modulus_size,
      public_exponent=big_integer_util.num_to_bytes(public_exponent))
  key_template = tink_pb2.KeyTemplate(
      value=key_format.SerializeToString(),
      type_url=_RSA_PSS_KEY_TYPE_URL,
      output_prefix_type=tink_pb2.TINK)

  return key_template


ECDSA_P256 = _create_ecdsa_key_template(common_pb2.SHA256, common_pb2.NIST_P256,
                                        ecdsa_pb2.DER)
ECDSA_P256_RAW = _create_ecdsa_key_template(common_pb2.SHA256,
                                            common_pb2.NIST_P256,
                                            ecdsa_pb2.IEEE_P1363, tink_pb2.RAW)
# Don't use this template, because it's name is confusing.
# Use ECDSA_P384_SHA512 instead, which is the same template.
ECDSA_P384 = _create_ecdsa_key_template(common_pb2.SHA512, common_pb2.NIST_P384,
                                        ecdsa_pb2.DER)
ECDSA_P384_SHA384 = _create_ecdsa_key_template(common_pb2.SHA384,
                                               common_pb2.NIST_P384,
                                               ecdsa_pb2.DER)
ECDSA_P384_SHA512 = _create_ecdsa_key_template(common_pb2.SHA512,
                                               common_pb2.NIST_P384,
                                               ecdsa_pb2.DER)
ECDSA_P521 = _create_ecdsa_key_template(common_pb2.SHA512, common_pb2.NIST_P521,
                                        ecdsa_pb2.DER)

# Deprecated. This key template does not make sense because IEEE P1363 mandates
# a raw signature.
ECDSA_P256_IEEE_P1363 = _create_ecdsa_key_template(common_pb2.SHA256,
                                                   common_pb2.NIST_P256,
                                                   ecdsa_pb2.IEEE_P1363)
# Deprecated. This key template does not make sense because IEEE P1363 mandates
# a raw signature.
ECDSA_P384_IEEE_P1363 = _create_ecdsa_key_template(common_pb2.SHA512,
                                                   common_pb2.NIST_P384,
                                                   ecdsa_pb2.IEEE_P1363)
# Deprecated. This key template does not make sense because IEEE P1363 mandates
# a raw signature.
ECDSA_P384_SHA384_IEEE_P1363 = _create_ecdsa_key_template(
    common_pb2.SHA384, common_pb2.NIST_P384, ecdsa_pb2.IEEE_P1363)
# Deprecated. This key template does not make sense because IEEE P1363 mandates
# a raw signature.
ECDSA_P521_IEEE_P1363 = _create_ecdsa_key_template(common_pb2.SHA512,
                                                   common_pb2.NIST_P521,
                                                   ecdsa_pb2.IEEE_P1363)

ED25519 = tink_pb2.KeyTemplate(
    type_url=_ED25519_KEY_TYPE_URL, output_prefix_type=tink_pb2.TINK)

RSA_SSA_PKCS1_3072_SHA256_F4 = _create_rsa_ssa_pkcs1_key_template(
    common_pb2.SHA256, 3072, _RSA_F4)
RSA_SSA_PKCS1_4096_SHA512_F4 = _create_rsa_ssa_pkcs1_key_template(
    common_pb2.SHA512, 4096, _RSA_F4)

RSA_SSA_PSS_3072_SHA256_SHA256_32_F4 = _create_rsa_ssa_pss_key_template(
    common_pb2.SHA256, common_pb2.SHA256, 32, 3072, _RSA_F4)
RSA_SSA_PSS_4096_SHA512_SHA512_64_F4 = _create_rsa_ssa_pss_key_template(
    common_pb2.SHA512, common_pb2.SHA512, 64, 4096, _RSA_F4)


# Deprecated. Use the predefined constant templates above instead.
def create_ecdsa_key_template(
    hash_type: common_pb2.HashType,
    curve: common_pb2.EllipticCurveType,
    encoding: ecdsa_pb2.EcdsaSignatureEncoding,
    output_prefix_type: tink_pb2.OutputPrefixType = tink_pb2.TINK
) -> tink_pb2.KeyTemplate:
  warnings.warn('The "create_ecdsa_key_template" function is deprecated.',
                DeprecationWarning, 2)
  return _create_ecdsa_key_template(hash_type, curve, encoding,
                                    output_prefix_type)


# Deprecated. Use the predefined constant templates above instead.
def create_rsa_ssa_pkcs1_key_template(
    hash_type: common_pb2.HashType, modulus_size: int,
    public_exponent: int) -> tink_pb2.KeyTemplate:
  warnings.warn(
      'The "create_rsa_ssa_pkcs1_key_template" function is deprecated.',
      DeprecationWarning, 2)
  return _create_rsa_ssa_pkcs1_key_template(hash_type, modulus_size,
                                            public_exponent)


# Deprecated. Use the predefined constant templates above instead.
def create_rsa_ssa_pss_key_template(
    sig_hash: common_pb2.HashType, mgf1_hash: common_pb2.HashType,
    salt_length: int, modulus_size: int,
    public_exponent: int) -> tink_pb2.KeyTemplate:
  warnings.warn('The "create_rsa_ssa_pss_key_template" function is deprecated.',
                DeprecationWarning, 2)
  return _create_rsa_ssa_pss_key_template(sig_hash, mgf1_hash, salt_length,
                                          modulus_size, public_exponent)



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
"""The verified JSON Web Token (JWT)."""

import datetime

from typing import List, Set

from tink import core
from tink.jwt import _raw_jwt


class VerifiedJwt:
  """A decoded and verified JSON Web Token (JWT).

  A new instance of this class is returned as the result of a sucessfully
  verification of a MACed or signed compact JWT.

  It gives read-only access all payload claims and a subset of the headers. It
  does not contain any headers that depend on the key, such as "alg" or "kid".
  These headers are checked when the signature is verified and should not be
  read by the user. This ensures that the key can be changed without any changes
  to the user code.
  """

  def __new__(cls):
    raise core.TinkError('VerifiedJwt cannot be instantiated directly.')

  @classmethod
  def _create(cls, raw_jwt: _raw_jwt.RawJwt):
    o = object.__new__(cls)
    o.__init__(raw_jwt)
    return o

  def __init__(self, raw_jwt: _raw_jwt.RawJwt) -> None:
    self._raw_jwt = raw_jwt

  def has_type_header(self) -> bool:
    return self._raw_jwt.has_type_header()

  def type_header(self) -> str:
    return self._raw_jwt.type_header()

  def has_issuer(self) -> bool:
    return self._raw_jwt.has_issuer()

  def issuer(self) -> str:
    return self._raw_jwt.issuer()

  def has_subject(self) -> bool:
    return self._raw_jwt.has_subject()

  def subject(self) -> str:
    return self._raw_jwt.subject()

  def has_audiences(self) -> bool:
    return self._raw_jwt.has_audiences()

  def audiences(self) -> List[str]:
    return self._raw_jwt.audiences()

  def has_jwt_id(self) -> bool:
    return self._raw_jwt.has_jwt_id()

  def jwt_id(self) -> str:
    return self._raw_jwt.jwt_id()

  def has_expiration(self) -> bool:
    return self._raw_jwt.has_expiration()

  def expiration(self) -> datetime.datetime:
    return self._raw_jwt.expiration()

  def has_not_before(self) -> bool:
    return self._raw_jwt.has_not_before()

  def not_before(self) -> datetime.datetime:
    return self._raw_jwt.not_before()

  def has_issued_at(self) -> bool:
    return self._raw_jwt.has_issued_at()

  def issued_at(self) -> datetime.datetime:
    return self._raw_jwt.issued_at()

  def custom_claim_names(self) -> Set[str]:
    return self._raw_jwt.custom_claim_names()

  def custom_claim(self, name: str) -> _raw_jwt.Claim:
    return self._raw_jwt.custom_claim(name)



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.public_key_sign_wrapper."""

from absl.testing import absltest
from absl.testing import parameterized

import tink
from tink import _keyset_handle
from tink import _monitoring
from tink import core
from tink import signature
from tink.testing import fake_key_usage_monitor
from tink.testing import keyset_builder


TEMPLATE = signature.signature_key_templates.ECDSA_P256
LEGACY_TEMPLATE = keyset_builder.legacy_template(TEMPLATE)
RAW_TEMPLATE = keyset_builder.raw_template(TEMPLATE)


def setUpModule():
  signature.register()


class SignatureWrapperTest(parameterized.TestCase):

  @parameterized.parameters([TEMPLATE, LEGACY_TEMPLATE, RAW_TEMPLATE])
  def test_sign_verify(self, template):
    private_handle = tink.new_keyset_handle(template)
    public_handle = private_handle.public_keyset_handle()
    sign_primitive = private_handle.primitive(signature.PublicKeySign)
    verify_primitive = public_handle.primitive(signature.PublicKeyVerify)

    data_signature = sign_primitive.sign(b'data')
    verify_primitive.verify(data_signature, b'data')

  @parameterized.parameters([TEMPLATE, LEGACY_TEMPLATE, RAW_TEMPLATE])
  def test_verify_fails_on_wrong_data(self, template):
    private_handle = tink.new_keyset_handle(template)
    public_handle = private_handle.public_keyset_handle()
    sign_primitive = private_handle.primitive(signature.PublicKeySign)
    verify_primitive = public_handle.primitive(signature.PublicKeyVerify)

    data_signature = sign_primitive.sign(b'data')
    with self.assertRaises(tink.TinkError):
      verify_primitive.verify(data_signature, b'invalid data')

  @parameterized.parameters([TEMPLATE, LEGACY_TEMPLATE, RAW_TEMPLATE])
  def test_verify_fails_on_unknown_signature(self, template):
    unknown_handle = tink.new_keyset_handle(template)
    unknown_sign_primitive = unknown_handle.primitive(signature.PublicKeySign)
    unknown_data_signature = unknown_sign_primitive.sign(b'data')

    private_handle = tink.new_keyset_handle(template)
    public_handle = private_handle.public_keyset_handle()
    verify_primitive = public_handle.primitive(signature.PublicKeyVerify)
    with self.assertRaises(tink.TinkError):
      verify_primitive.verify(unknown_data_signature, b'data')

  @parameterized.parameters([
      (TEMPLATE, TEMPLATE),
      (TEMPLATE, LEGACY_TEMPLATE),
      (TEMPLATE, RAW_TEMPLATE),
      (LEGACY_TEMPLATE, TEMPLATE),
      (LEGACY_TEMPLATE, LEGACY_TEMPLATE),
      (LEGACY_TEMPLATE, RAW_TEMPLATE),
      (RAW_TEMPLATE, TEMPLATE),
      (RAW_TEMPLATE, LEGACY_TEMPLATE),
      (RAW_TEMPLATE, RAW_TEMPLATE),
  ])
  def test_sign_verify_with_key_rotation(self, old_template, new_template):
    builder = keyset_builder.new_keyset_builder()
    older_key_id = builder.add_new_key(old_template)
    builder.set_primary_key(older_key_id)
    private_handle1 = builder.keyset_handle()
    sign1 = private_handle1.primitive(signature.PublicKeySign)
    verify1 = private_handle1.public_keyset_handle().primitive(
        signature.PublicKeyVerify
    )

    newer_key_id = builder.add_new_key(new_template)
    private_handle2 = builder.keyset_handle()
    sign2 = private_handle2.primitive(signature.PublicKeySign)
    verify2 = private_handle2.public_keyset_handle().primitive(
        signature.PublicKeyVerify
    )

    builder.set_primary_key(newer_key_id)
    private_handle3 = builder.keyset_handle()
    sign3 = private_handle3.primitive(signature.PublicKeySign)
    verify3 = private_handle3.public_keyset_handle().primitive(
        signature.PublicKeyVerify
    )

    builder.disable_key(older_key_id)
    private_handle4 = builder.keyset_handle()
    sign4 = private_handle4.primitive(signature.PublicKeySign)
    verify4 = private_handle4.public_keyset_handle().primitive(
        signature.PublicKeyVerify
    )
    self.assertNotEqual(older_key_id, newer_key_id)

    # 1 signs with the older key. So 1, 2 and 3 can verify it, but not 4.
    data_signature1 = sign1.sign(b'data')
    verify1.verify(data_signature1, b'data')
    verify2.verify(data_signature1, b'data')
    verify3.verify(data_signature1, b'data')
    with self.assertRaises(tink.TinkError):
      verify4.verify(data_signature1, b'data')

    # 2 signs with the older key. So 1, 2 and 3 can verify it, but not 4.
    data_signature2 = sign2.sign(b'data')
    verify1.verify(data_signature2, b'data')
    verify2.verify(data_signature2, b'data')
    verify3.verify(data_signature2, b'data')
    with self.assertRaises(tink.TinkError):
      verify4.verify(data_signature2, b'data')

    # 3 signs with the newer key. So 2, 3 and 4 can verify it, but not 1.
    data_signature3 = sign3.sign(b'data')
    with self.assertRaises(tink.TinkError):
      verify1.verify(data_signature3, b'data')
    verify2.verify(data_signature3, b'data')
    verify3.verify(data_signature3, b'data')
    verify4.verify(data_signature3, b'data')

    # 4 signs with the newer key. So 2, 3 and 4 can verify it, but not 1.
    data_signature4 = sign4.sign(b'data')
    with self.assertRaises(tink.TinkError):
      verify1.verify(data_signature4, b'data')
    verify2.verify(data_signature4, b'data')
    verify3.verify(data_signature4, b'data')
    verify4.verify(data_signature4, b'data')


class KeyUsageMonitorTest(absltest.TestCase):

  def setUp(self):
    super().setUp()
    self.sign_key_usage_monitor = fake_key_usage_monitor.FakeKeyUsageMonitor()
    self.verify_key_usage_monitor = fake_key_usage_monitor.FakeKeyUsageMonitor()
    _monitoring.register_key_usage_monitor_factory(
        lambda context: (
            self.sign_key_usage_monitor
            if context.get_api_function() == 'sign'
            else self.verify_key_usage_monitor
        )
    )

  def test_key_usage_monitor_log(self):
    private_handle = _keyset_handle._new_keyset_handle_with_annotations(
        TEMPLATE, {'test': 'test'}
    )
    public_handle = private_handle.public_keyset_handle()
    sign_primitive = private_handle.primitive(signature.PublicKeySign)
    verify_primitive = public_handle.primitive(signature.PublicKeyVerify)

    data_signature = sign_primitive.sign(b'data')
    verify_primitive.verify(data_signature, b'data')

    self.assertSequenceEqual(
        self.sign_key_usage_monitor.log_calls,
        [(private_handle.keyset_info().key_info[0].key_id, len(b'data'))],
    )
    self.assertSequenceEqual(
        self.verify_key_usage_monitor.log_calls,
        [(
            public_handle.keyset_info().key_info[0].key_id,
            len(data_signature),
        )],
    )

  def test_key_usage_monitor_log_failure(self):
    private_handle = _keyset_handle._new_keyset_handle_with_annotations(
        TEMPLATE, {'test': 'test'}
    )
    public_handle = private_handle.public_keyset_handle()
    verify_primitive = public_handle.primitive(signature.PublicKeyVerify)

    try:
      verify_primitive.verify(b'x', b'data')
    except core.TinkError:
      pass

    self.assertEqual(self.verify_key_usage_monitor.log_failure_calls_count, 1)


if __name__ == '__main__':
  absltest.main()
